@@ -3,8 +3,10 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
+#include "ErrorList.h"
 #include "HTMLEditor.h"
 
+#include "CSSEditUtils.h"
 #include "HTMLEditorEventListener.h"
 #include "HTMLEditUtils.h"
 
@@ -323,7 +325,9 @@ nsresult HTMLEditor::ShowResizersInternal(Element& aResizedElement) {
     return NS_ERROR_FAILURE;
   }
 
-  if (NS_WARN_IF(!IsDescendantOfEditorRoot(&aResizedElement))) {
+  const RefPtr<Element> editingHost = ComputeEditingHost();
+  if (NS_WARN_IF(!editingHost) ||
+      NS_WARN_IF(!aResizedElement.IsInclusiveDescendantOf(editingHost))) {
     return NS_ERROR_UNEXPECTED;
   }
 
@@ -672,8 +676,7 @@ nsresult HTMLEditor::StartResizing(Element& aHandleElement) {
       "Element::SetAttr(nsGkAtoms::_moz_activated, true) failed");
 
   // do we want to preserve ratio or not?
-  const bool preserveRatio = StaticPrefs::editor_resizing_preserve_ratio() &&
-                             HTMLEditUtils::IsImage(mResizedObject);
+  const bool preserveRatio = HTMLEditUtils::IsImage(mResizedObject);
 
   // the way we change the position/size of the shadow depends on
   // the handle
@@ -710,7 +713,7 @@ nsresult HTMLEditor::StartResizing(Element& aHandleElement) {
   if (RefPtr<nsStyledElement> resizingShadowStyledElement =
           nsStyledElement::FromNodeOrNull(mResizingShadow.get())) {
     nsresult rv;
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
         *resizingShadowStyledElement, *nsGkAtoms::width, mResizedObjectWidth);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
@@ -721,7 +724,7 @@ nsresult HTMLEditor::StartResizing(Element& aHandleElement) {
     NS_WARNING_ASSERTION(NS_SUCCEEDED(rv),
                          "CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction("
                          "nsGkAtoms::width) failed");
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
         *resizingShadowStyledElement, *nsGkAtoms::height, mResizedObjectHeight);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
       NS_WARNING(
@@ -903,7 +906,7 @@ nsresult HTMLEditor::SetResizingInfoPosition(int32_t aX, int32_t aY, int32_t aW,
   if (RefPtr<nsStyledElement> resizingInfoStyledElement =
           nsStyledElement::FromNodeOrNull(mResizingInfo.get())) {
     nsresult rv;
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
         *resizingInfoStyledElement, *nsGkAtoms::left,
         infoXPosition + mouseCursorOffset);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
@@ -916,7 +919,7 @@ nsresult HTMLEditor::SetResizingInfoPosition(int32_t aX, int32_t aY, int32_t aW,
         NS_SUCCEEDED(rv),
         "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::left) "
         "failed, but ignored");
-    rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+    rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
         *resizingInfoStyledElement, *nsGkAtoms::top,
         infoYPosition + mouseCursorOffset);
     if (rv == NS_ERROR_EDITOR_DESTROYED) {
@@ -1110,7 +1113,7 @@ nsresult HTMLEditor::UpdateResizerOrGrabberPositionTo(
     if (RefPtr<nsStyledElement> resizingShadowStyledElement =
             nsStyledElement::FromNodeOrNull(mResizingShadow.get())) {
       nsresult rv;
-      rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+      rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
           *resizingShadowStyledElement, *nsGkAtoms::left, newX);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
@@ -1122,7 +1125,7 @@ nsresult HTMLEditor::UpdateResizerOrGrabberPositionTo(
           NS_SUCCEEDED(rv),
           "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::left) "
           "failed, but ignored");
-      rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+      rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
           *resizingShadowStyledElement, *nsGkAtoms::top, newY);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
@@ -1134,7 +1137,7 @@ nsresult HTMLEditor::UpdateResizerOrGrabberPositionTo(
           NS_SUCCEEDED(rv),
           "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::top) "
           "failed, but ignored");
-      rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+      rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
           *resizingShadowStyledElement, *nsGkAtoms::width, newWidth);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
@@ -1146,7 +1149,7 @@ nsresult HTMLEditor::UpdateResizerOrGrabberPositionTo(
           NS_SUCCEEDED(rv),
           "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::width) "
           "failed, but ignored");
-      rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+      rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
           *resizingShadowStyledElement, *nsGkAtoms::height, newHeight);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
@@ -1195,7 +1198,7 @@ nsresult HTMLEditor::UpdateResizerOrGrabberPositionTo(
     if (RefPtr<nsStyledElement> positioningShadowStyledElement =
             nsStyledElement::FromNodeOrNull(mPositioningShadow.get())) {
       nsresult rv;
-      rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+      rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
           *positioningShadowStyledElement, *nsGkAtoms::left, newX);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
@@ -1207,7 +1210,7 @@ nsresult HTMLEditor::UpdateResizerOrGrabberPositionTo(
           NS_SUCCEEDED(rv),
           "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::left) "
           "failed, but ignored");
-      rv = mCSSEditUtils->SetCSSPropertyPixelsWithoutTransaction(
+      rv = CSSEditUtils::SetCSSPropertyPixelsWithoutTransaction(
           *positioningShadowStyledElement, *nsGkAtoms::top, newY);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
@@ -1268,8 +1271,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
 
   if (mResizedObjectIsAbsolutelyPositioned && resizedStyleElement) {
     if (setHeight) {
-      nsresult rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-          *resizedStyleElement, *nsGkAtoms::top, y);
+      nsresult rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+          *this, *resizedStyleElement, *nsGkAtoms::top, y);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
             "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::"
@@ -1282,8 +1285,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
           "failed, but ignored");
     }
     if (setWidth) {
-      nsresult rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-          *resizedStyleElement, *nsGkAtoms::left, x);
+      nsresult rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+          *this, *resizedStyleElement, *nsGkAtoms::left, x);
       if (rv == NS_ERROR_EDITOR_DESTROYED) {
         NS_WARNING(
             "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::"
@@ -1299,34 +1302,40 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
   if (IsCSSEnabled() || mResizedObjectIsAbsolutelyPositioned) {
     if (setWidth &&
         resizedElement->HasAttr(kNameSpaceID_None, nsGkAtoms::width)) {
-      DebugOnly<nsresult> rvIgnored =
+      nsresult rv =
           RemoveAttributeWithTransaction(*resizedElement, *nsGkAtoms::width);
-      if (NS_WARN_IF(Destroyed())) {
+      if (MOZ_UNLIKELY(rv == NS_ERROR_EDITOR_DESTROYED)) {
+        NS_WARNING(
+            "EditorBase::RemoveAttributeWithTransaction(nsGkAtoms::width) "
+            "failed");
         return NS_ERROR_EDITOR_DESTROYED;
       }
       NS_WARNING_ASSERTION(
-          NS_SUCCEEDED(rvIgnored),
+          NS_SUCCEEDED(rv),
           "EditorBase::RemoveAttributeWithTransaction(nsGkAtoms::width) "
           "failed, but ignored");
     }
 
     if (setHeight &&
         resizedElement->HasAttr(kNameSpaceID_None, nsGkAtoms::height)) {
-      DebugOnly<nsresult> rvIgnored =
+      nsresult rv =
           RemoveAttributeWithTransaction(*resizedElement, *nsGkAtoms::height);
-      if (NS_WARN_IF(Destroyed())) {
+      if (MOZ_UNLIKELY(rv == NS_ERROR_EDITOR_DESTROYED)) {
+        NS_WARNING(
+            "EditorBase::RemoveAttributeWithTransaction(nsGkAtoms::height) "
+            "failed");
         return NS_ERROR_EDITOR_DESTROYED;
       }
       NS_WARNING_ASSERTION(
-          NS_SUCCEEDED(rvIgnored),
+          NS_SUCCEEDED(rv),
           "EditorBase::RemoveAttributeWithTransaction(nsGkAtoms::height) "
           "failed, but ignored");
     }
 
     if (resizedStyleElement) {
       if (setWidth) {
-        nsresult rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-            *resizedStyleElement, *nsGkAtoms::width, width);
+        nsresult rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+            *this, *resizedStyleElement, *nsGkAtoms::width, width);
         if (NS_FAILED(rv)) {
           NS_WARNING(
               "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::"
@@ -1339,8 +1348,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
             "failed, but ignored");
       }
       if (setHeight) {
-        nsresult rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-            *resizedStyleElement, *nsGkAtoms::height, height);
+        nsresult rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+            *this, *resizedStyleElement, *nsGkAtoms::height, height);
         if (NS_FAILED(rv)) {
           NS_WARNING(
               "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::"
@@ -1361,8 +1370,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
     // with asynchronous reflow
     if (resizedStyleElement) {
       if (setWidth) {
-        nsresult rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-            *resizedStyleElement, *nsGkAtoms::width, width);
+        nsresult rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+            *this, *resizedStyleElement, *nsGkAtoms::width, width);
         if (NS_FAILED(rv)) {
           NS_WARNING(
               "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::"
@@ -1375,8 +1384,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
             "width) failed, but ignored");
       }
       if (setHeight) {
-        nsresult rv = mCSSEditUtils->SetCSSPropertyPixelsWithTransaction(
-            *resizedStyleElement, *nsGkAtoms::height, height);
+        nsresult rv = CSSEditUtils::SetCSSPropertyPixelsWithTransaction(
+            *this, *resizedStyleElement, *nsGkAtoms::height, height);
         if (NS_FAILED(rv)) {
           NS_WARNING(
               "CSSEditUtils::SetCSSPropertyPixelsWithTransaction(nsGkAtoms::"
@@ -1418,8 +1427,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
 
     if (resizedStyleElement) {
       if (setWidth) {
-        nsresult rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-            *resizedStyleElement, *nsGkAtoms::width, u""_ns);
+        nsresult rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+            *this, *resizedStyleElement, *nsGkAtoms::width, u""_ns);
         if (rv == NS_ERROR_EDITOR_DESTROYED) {
           NS_WARNING(
               "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::width)"
@@ -1432,8 +1441,8 @@ nsresult HTMLEditor::SetFinalSizeWithTransaction(int32_t aX, int32_t aY) {
             "failed, but ignored");
       }
       if (setHeight) {
-        nsresult rv = mCSSEditUtils->RemoveCSSPropertyWithTransaction(
-            *resizedStyleElement, *nsGkAtoms::height, u""_ns);
+        nsresult rv = CSSEditUtils::RemoveCSSPropertyWithTransaction(
+            *this, *resizedStyleElement, *nsGkAtoms::height, u""_ns);
         if (rv == NS_ERROR_EDITOR_DESTROYED) {
           NS_WARNING(
               "CSSEditUtils::RemoveCSSPropertyWithTransaction(nsGkAtoms::"
