@@ -8,21 +8,21 @@ const knownFailures = {
     "A-Proposed-CB:name_TEXT-1_SI-dM": true,
     "A-Proposed-CB:name_TEXT-1_SI-body": true,
     "A-Proposed-CB:name_TEXT-1_SI-div": true,
-    "A-Proposed-DECFS:2_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "A-Proposed-DECFS:2_TEXT-1_SI-body": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "A-Proposed-DECFS:2_TEXT-1_SI-div": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
+    "A-Proposed-DECFS:2_TEXT-1_SI-dM": true,
+    "A-Proposed-DECFS:2_TEXT-1_SI-body": true,
+    "A-Proposed-DECFS:2_TEXT-1_SI-div": true,
     "A-Proposed-FS:18px_TEXT-1_SI-dM": true,
     "A-Proposed-FS:18px_TEXT-1_SI-body": true,
     "A-Proposed-FS:18px_TEXT-1_SI-div": true,
     "A-Proposed-FS:large_TEXT-1_SI-dM": true,
     "A-Proposed-FS:large_TEXT-1_SI-body": true,
     "A-Proposed-FS:large_TEXT-1_SI-div": true,
-    "A-Proposed-H:H1_TEXT-1_SC-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "A-Proposed-H:H1_TEXT-1_SC-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "A-Proposed-H:H1_TEXT-1_SC-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "A-Proposed-INCFS:2_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
-    "A-Proposed-INCFS:2_TEXT-1_SI-body": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
-    "A-Proposed-INCFS:2_TEXT-1_SI-div": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
+    "A-Proposed-H:H1_TEXT-1_SC-dM": true,
+    "A-Proposed-H:H1_TEXT-1_SC-body": true,
+    "A-Proposed-H:H1_TEXT-1_SC-div": true,
+    "A-Proposed-INCFS:2_TEXT-1_SI-dM": true,
+    "A-Proposed-INCFS:2_TEXT-1_SI-body": true,
+    "A-Proposed-INCFS:2_TEXT-1_SI-div": true,
     "AC-Proposed-SUB_TEXT-1_SI-dM": true,
     "AC-Proposed-SUB_TEXT-1_SI-body": true,
     "AC-Proposed-SUB_TEXT-1_SI-div": true,
@@ -38,24 +38,23 @@ const knownFailures = {
     "AC-Proposed-FS:large_TEXT-1_SI-dM": true,
     "AC-Proposed-FS:large_TEXT-1_SI-body": true,
     "AC-Proposed-FS:large_TEXT-1_SI-div": true,
-    "C-Proposed-BC:ace_FONT.ass.s:bc:rgb-1_SW-dM": true,
-    "C-Proposed-BC:ace_FONT.ass.s:bc:rgb-1_SW-body": true,
-    "C-Proposed-BC:ace_FONT.ass.s:bc:rgb-1_SW-div": true,
-    "C-Proposed-HC:g_SPAN.ass.s:c:rgb-1_SW-dM": true,
-    "C-Proposed-HC:g_SPAN.ass.s:c:rgb-1_SW-body": true,
-    "C-Proposed-HC:g_SPAN.ass.s:c:rgb-1_SW-div": true,
-    "C-Proposed-FN:c_FONTf:a-1_SI-dM": true,
-    "C-Proposed-FN:c_FONTf:a-1_SI-body": true,
-    "C-Proposed-FN:c_FONTf:a-1_SI-div": true,
-    "C-Proposed-FN:c_FONTf:a-2_SL-dM": true,
-    "C-Proposed-FN:c_FONTf:a-2_SL-body": true,
-    "C-Proposed-FN:c_FONTf:a-2_SL-div": true,
+
+    // Those tests expect that <font> elements can be nested, but they don't
+    // match with the other browsers' behavior.
+    "C-Proposed-FC:g_FONTc:b.sz:6-1_SI-dM": true,
+    "C-Proposed-FC:g_FONTc:b.sz:6-1_SI-body": true,
+    "C-Proposed-FC:g_FONTc:b.sz:6-1_SI-div": true,
+
     "C-Proposed-FS:1_SPAN.ass.s:fs:large-1_SW-dM": true,
     "C-Proposed-FS:1_SPAN.ass.s:fs:large-1_SW-body": true,
     "C-Proposed-FS:1_SPAN.ass.s:fs:large-1_SW-div": true,
-    "C-Proposed-FS:5_FONTsz:1.s:fs:xs-1_SW-dM": true,
-    "C-Proposed-FS:5_FONTsz:1.s:fs:xs-1_SW-body": true,
-    "C-Proposed-FS:5_FONTsz:1.s:fs:xs-1_SW-div": true,
+
+    // Those tests expect that <font> elements can be nested, but they don't
+    // match with the other browsers' behavior.
+    "C-Proposed-FS:2_FONTc:b.sz:6-1_SI-dM": true,
+    "C-Proposed-FS:2_FONTc:b.sz:6-1_SI-body": true,
+    "C-Proposed-FS:2_FONTc:b.sz:6-1_SI-div": true,
+
     "C-Proposed-FS:larger_FONTsz:4-dM": true,
     "C-Proposed-FS:larger_FONTsz:4-body": true,
     "C-Proposed-FS:larger_FONTsz:4-div": true,
@@ -86,12 +85,6 @@ const knownFailures = {
     "CC-Proposed-BC:gray_SPANs:bc:b-2_SR-dM": true,
     "CC-Proposed-BC:gray_SPANs:bc:b-2_SR-body": true,
     "CC-Proposed-BC:gray_SPANs:bc:b-2_SR-div": true,
-    "CC-Proposed-FN:c_FONTf:a-1_SI-dM": true,
-    "CC-Proposed-FN:c_FONTf:a-1_SI-body": true,
-    "CC-Proposed-FN:c_FONTf:a-1_SI-div": true,
-    "CC-Proposed-FN:c_FONTf:a-2_SL-dM": true,
-    "CC-Proposed-FN:c_FONTf:a-2_SL-body": true,
-    "CC-Proposed-FN:c_FONTf:a-2_SL-div": true,
     "CC-Proposed-FS:1_SPANs:fs:l-1_SW-dM": true,
     "CC-Proposed-FS:1_SPANs:fs:l-1_SW-body": true,
     "CC-Proposed-FS:1_SPANs:fs:l-1_SW-div": true,
@@ -279,15 +272,15 @@ const knownFailures = {
     "Q-Proposed-CREATEBOOKMARK_TEXT-1-dM": true,
     "Q-Proposed-CREATEBOOKMARK_TEXT-1-body": true,
     "Q-Proposed-CREATEBOOKMARK_TEXT-1-div": true,
-    "Q-Proposed-DECREASEFONTSIZE_TEXT-1-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "Q-Proposed-DECREASEFONTSIZE_TEXT-1-body": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "Q-Proposed-DECREASEFONTSIZE_TEXT-1-div": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "Q-Proposed-HEADING_TEXT-1-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "Q-Proposed-HEADING_TEXT-1-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "Q-Proposed-HEADING_TEXT-1-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "Q-Proposed-INCREASEFONTSIZE_TEXT-1-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
-    "Q-Proposed-INCREASEFONTSIZE_TEXT-1-body": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
-    "Q-Proposed-INCREASEFONTSIZE_TEXT-1-div": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
+    "Q-Proposed-DECREASEFONTSIZE_TEXT-1-dM": true,
+    "Q-Proposed-DECREASEFONTSIZE_TEXT-1-body": true,
+    "Q-Proposed-DECREASEFONTSIZE_TEXT-1-div": true,
+    "Q-Proposed-HEADING_TEXT-1-dM": true,
+    "Q-Proposed-HEADING_TEXT-1-body": true,
+    "Q-Proposed-HEADING_TEXT-1-div": true,
+    "Q-Proposed-INCREASEFONTSIZE_TEXT-1-dM": true,
+    "Q-Proposed-INCREASEFONTSIZE_TEXT-1-body": true,
+    "Q-Proposed-INCREASEFONTSIZE_TEXT-1-div": true,
     "Q-Proposed-PASTE_TEXT-1-dM": true,
     "Q-Proposed-PASTE_TEXT-1-body": true,
     "Q-Proposed-PASTE_TEXT-1-div": true,
@@ -309,15 +302,15 @@ const knownFailures = {
     "QE-Proposed-CUT_TEXT-1-dM": true,
     "QE-Proposed-CUT_TEXT-1-body": true,
     "QE-Proposed-CUT_TEXT-1-div": true,
-    "QE-Proposed-DECREASEFONTSIZE_TEXT-1-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "QE-Proposed-DECREASEFONTSIZE_TEXT-1-body": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "QE-Proposed-DECREASEFONTSIZE_TEXT-1-div": !SpecialPowers.getBoolPref("dom.document.edit_command.decreasefontsize.enabled", false),
-    "QE-Proposed-HEADING_TEXT-1-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QE-Proposed-HEADING_TEXT-1-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QE-Proposed-HEADING_TEXT-1-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QE-Proposed-INCREASEFONTSIZE_TEXT-1-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
-    "QE-Proposed-INCREASEFONTSIZE_TEXT-1-body": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
-    "QE-Proposed-INCREASEFONTSIZE_TEXT-1-div": !SpecialPowers.getBoolPref("dom.document.edit_command.increasefontsize.enabled", false),
+    "QE-Proposed-DECREASEFONTSIZE_TEXT-1-dM": true,
+    "QE-Proposed-DECREASEFONTSIZE_TEXT-1-body": true,
+    "QE-Proposed-DECREASEFONTSIZE_TEXT-1-div": true,
+    "QE-Proposed-HEADING_TEXT-1-dM": true,
+    "QE-Proposed-HEADING_TEXT-1-body": true,
+    "QE-Proposed-HEADING_TEXT-1-div": true,
+    "QE-Proposed-INCREASEFONTSIZE_TEXT-1-dM": true,
+    "QE-Proposed-INCREASEFONTSIZE_TEXT-1-body": true,
+    "QE-Proposed-INCREASEFONTSIZE_TEXT-1-div": true,
     "QE-Proposed-PASTE_TEXT-1-dM": true,
     "QE-Proposed-PASTE_TEXT-1-body": true,
     "QE-Proposed-PASTE_TEXT-1-div": true,
@@ -432,18 +425,18 @@ const knownFailures = {
     "QV-Proposed-FB_H1-H2.TEXT.H2-1_SM-dM": true,
     "QV-Proposed-FB_H1-H2.TEXT.H2-1_SM-body": true,
     "QV-Proposed-FB_H1-H2.TEXT.H2-1_SM-div": true,
-    "QV-Proposed-H_H1-1_SC-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H1-1_SC-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H1-1_SC-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H3-1_SC-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H3-1_SC-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H3-1_SC-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H1-H2-H3-H4-1_SC-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H1-H2-H3-H4-1_SC-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_H1-H2-H3-H4-1_SC-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_P-1_SC-dM": SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_P-1_SC-body": SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "QV-Proposed-H_P-1_SC-div": SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
+    "QV-Proposed-H_H1-1_SC-dM": true,
+    "QV-Proposed-H_H1-1_SC-body": true,
+    "QV-Proposed-H_H1-1_SC-div": true,
+    "QV-Proposed-H_H3-1_SC-dM": true,
+    "QV-Proposed-H_H3-1_SC-body": true,
+    "QV-Proposed-H_H3-1_SC-div": true,
+    "QV-Proposed-H_H1-H2-H3-H4-1_SC-dM": true,
+    "QV-Proposed-H_H1-H2-H3-H4-1_SC-body": true,
+    "QV-Proposed-H_H1-H2-H3-H4-1_SC-div": true,
+    "QV-Proposed-H_P-1_SC-dM": false,
+    "QV-Proposed-H_P-1_SC-body": false,
+    "QV-Proposed-H_P-1_SC-div": false,
     "QV-Proposed-FS_FONTs:fs:l-1_SI-dM": true,
     "QV-Proposed-FS_FONTs:fs:l-1_SI-body": true,
     "QV-Proposed-FS_FONTs:fs:l-1_SI-div": true,
@@ -569,48 +562,48 @@ const knownFailures = {
     "S-Proposed-SM:e.f.l_BR.BR-2_SI-1-dM": true,
     "S-Proposed-SM:e.f.l_BR.BR-2_SI-1-body": true,
     "S-Proposed-SM:e.f.l_BR.BR-2_SI-1-div": true,
-    "A-Proposed-B_TEXT-1_SI-dM": true,
-    "A-Proposed-B_TEXT-1_SI-body": true,
-    "A-Proposed-B_TEXT-1_SI-div": true,
+    "A-Proposed-B_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-B_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-B_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "A-Proposed-B_TEXT-1_SIR-dM": true,
     "A-Proposed-B_TEXT-1_SIR-body": true,
     "A-Proposed-B_TEXT-1_SIR-div": true,
-    "A-Proposed-B_I-1_SL-dM": true,
-    "A-Proposed-B_I-1_SL-body": true,
-    "A-Proposed-B_I-1_SL-div": true,
-    "A-Proposed-I_TEXT-1_SI-dM": true,
-    "A-Proposed-I_TEXT-1_SI-body": true,
-    "A-Proposed-I_TEXT-1_SI-div": true,
-    "A-Proposed-U_TEXT-1_SI-dM": true,
-    "A-Proposed-U_TEXT-1_SI-body": true,
-    "A-Proposed-U_TEXT-1_SI-div": true,
-    "A-Proposed-S_TEXT-1_SI-dM": true,
-    "A-Proposed-S_TEXT-1_SI-body": true,
-    "A-Proposed-S_TEXT-1_SI-div": true,
-    "A-Proposed-SUB_TEXT-1_SI-dM": true,
-    "A-Proposed-SUB_TEXT-1_SI-body": true,
-    "A-Proposed-SUB_TEXT-1_SI-div": true,
-    "A-Proposed-SUP_TEXT-1_SI-dM": true,
-    "A-Proposed-SUP_TEXT-1_SI-body": true,
-    "A-Proposed-SUP_TEXT-1_SI-div": true,
-    "A-Proposed-CL:url_TEXT-1_SI-dM": true,
-    "A-Proposed-CL:url_TEXT-1_SI-body": true,
-    "A-Proposed-CL:url_TEXT-1_SI-div": true,
-    "A-Proposed-BC:blue_TEXT-1_SI-dM": true,
-    "A-Proposed-BC:blue_TEXT-1_SI-body": true,
-    "A-Proposed-BC:blue_TEXT-1_SI-div": true,
-    "A-Proposed-FC:blue_TEXT-1_SI-dM": true,
-    "A-Proposed-FC:blue_TEXT-1_SI-body": true,
-    "A-Proposed-FC:blue_TEXT-1_SI-div": true,
-    "A-Proposed-HC:blue_TEXT-1_SI-dM": true,
-    "A-Proposed-HC:blue_TEXT-1_SI-body": true,
-    "A-Proposed-HC:blue_TEXT-1_SI-div": true,
-    "A-Proposed-FN:a_TEXT-1_SI-dM": true,
-    "A-Proposed-FN:a_TEXT-1_SI-body": true,
-    "A-Proposed-FN:a_TEXT-1_SI-div": true,
-    "A-Proposed-FS:2_TEXT-1_SI-dM": true,
-    "A-Proposed-FS:2_TEXT-1_SI-body": true,
-    "A-Proposed-FS:2_TEXT-1_SI-div": true,
+    "A-Proposed-B_I-1_SL-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-B_I-1_SL-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-B_I-1_SL-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-I_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-I_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-I_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-U_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-U_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-U_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-S_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-S_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-S_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-SUB_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-SUB_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-SUB_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-SUP_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-SUP_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-SUP_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-CL:url_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-CL:url_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-CL:url_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-BC:blue_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-BC:blue_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-BC:blue_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FC:blue_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FC:blue_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FC:blue_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-HC:blue_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-HC:blue_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-HC:blue_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FN:a_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FN:a_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FN:a_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FS:2_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FS:2_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "A-Proposed-FS:2_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "A-Proposed-FS:18px_TEXT-1_SI-dM": true,
     "A-Proposed-FS:18px_TEXT-1_SI-body": true,
     "A-Proposed-FS:18px_TEXT-1_SI-div": true,
@@ -626,39 +619,39 @@ const knownFailures = {
     "A-Proposed-CB:name_TEXT-1_SI-dM": true,
     "A-Proposed-CB:name_TEXT-1_SI-body": true,
     "A-Proposed-CB:name_TEXT-1_SI-div": true,
-    "A-Proposed-H:H1_TEXT-1_SC-dM": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "A-Proposed-H:H1_TEXT-1_SC-body": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "A-Proposed-H:H1_TEXT-1_SC-div": !SpecialPowers.getBoolPref("dom.document.edit_command.heading.enabled", false),
-    "AC-Proposed-B_TEXT-1_SI-dM": true,
-    "AC-Proposed-B_TEXT-1_SI-body": true,
-    "AC-Proposed-B_TEXT-1_SI-div": true,
-    "AC-Proposed-I_TEXT-1_SI-dM": true,
-    "AC-Proposed-I_TEXT-1_SI-body": true,
-    "AC-Proposed-I_TEXT-1_SI-div": true,
-    "AC-Proposed-U_TEXT-1_SI-dM": true,
-    "AC-Proposed-U_TEXT-1_SI-body": true,
-    "AC-Proposed-U_TEXT-1_SI-div": true,
-    "AC-Proposed-S_TEXT-1_SI-dM": true,
-    "AC-Proposed-S_TEXT-1_SI-body": true,
-    "AC-Proposed-S_TEXT-1_SI-div": true,
+    "A-Proposed-H:H1_TEXT-1_SC-dM": true,
+    "A-Proposed-H:H1_TEXT-1_SC-body": true,
+    "A-Proposed-H:H1_TEXT-1_SC-div": true,
+    "AC-Proposed-B_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-B_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-B_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-I_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-I_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-I_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-U_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-U_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-U_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-S_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-S_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-S_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "AC-Proposed-SUB_TEXT-1_SI-dM": true,
     "AC-Proposed-SUB_TEXT-1_SI-body": true,
     "AC-Proposed-SUB_TEXT-1_SI-div": true,
     "AC-Proposed-SUP_TEXT-1_SI-dM": true,
     "AC-Proposed-SUP_TEXT-1_SI-body": true,
     "AC-Proposed-SUP_TEXT-1_SI-div": true,
-    "AC-Proposed-BC:blue_TEXT-1_SI-dM": true,
-    "AC-Proposed-BC:blue_TEXT-1_SI-body": true,
-    "AC-Proposed-BC:blue_TEXT-1_SI-div": true,
-    "AC-Proposed-FC:blue_TEXT-1_SI-dM": true,
-    "AC-Proposed-FC:blue_TEXT-1_SI-body": true,
-    "AC-Proposed-FC:blue_TEXT-1_SI-div": true,
-    "AC-Proposed-HC:blue_TEXT-1_SI-dM": true,
-    "AC-Proposed-HC:blue_TEXT-1_SI-body": true,
-    "AC-Proposed-HC:blue_TEXT-1_SI-div": true,
-    "AC-Proposed-FN:a_TEXT-1_SI-dM": true,
-    "AC-Proposed-FN:a_TEXT-1_SI-body": true,
-    "AC-Proposed-FN:a_TEXT-1_SI-div": true,
+    "AC-Proposed-BC:blue_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-BC:blue_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-BC:blue_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-FC:blue_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-FC:blue_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-FC:blue_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-HC:blue_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-HC:blue_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-HC:blue_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-FN:a_TEXT-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-FN:a_TEXT-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "AC-Proposed-FN:a_TEXT-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "AC-Proposed-FS:2_TEXT-1_SI-dM": true,
     "AC-Proposed-FS:2_TEXT-1_SI-body": true,
     "AC-Proposed-FS:2_TEXT-1_SI-div": true,
@@ -668,45 +661,38 @@ const knownFailures = {
     "AC-Proposed-FS:large_TEXT-1_SI-dM": true,
     "AC-Proposed-FS:large_TEXT-1_SI-body": true,
     "AC-Proposed-FS:large_TEXT-1_SI-div": true,
-    "C-Proposed-I_I-1_SL-dM": true,
-    "C-Proposed-I_I-1_SL-body": true,
-    "C-Proposed-I_I-1_SL-div": true,
-    "C-Proposed-I_B-I-1_SO-dM": true,
-    "C-Proposed-I_B-I-1_SO-body": true,
-    "C-Proposed-I_B-I-1_SO-div": true,
-    "C-Proposed-U_U-1_SO-dM": true,
-    "C-Proposed-U_U-1_SO-body": true,
-    "C-Proposed-U_U-1_SO-div": true,
-    "C-Proposed-U_U-1_SL-dM": true,
-    "C-Proposed-U_U-1_SL-body": true,
-    "C-Proposed-U_U-1_SL-div": true,
-    "C-Proposed-U_S-U-1_SO-dM": true,
-    "C-Proposed-U_S-U-1_SO-body": true,
-    "C-Proposed-U_S-U-1_SO-div": true,
-    "C-Proposed-BC:ace_FONT.ass.s:bc:rgb-1_SW-dM": true,
-    "C-Proposed-BC:ace_FONT.ass.s:bc:rgb-1_SW-body": true,
-    "C-Proposed-BC:ace_FONT.ass.s:bc:rgb-1_SW-div": true,
+    "C-Proposed-I_I-1_SL-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-I_I-1_SL-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-I_I-1_SL-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-I_B-I-1_SO-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-I_B-I-1_SO-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-I_B-I-1_SO-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_U-1_SO-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_U-1_SO-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_U-1_SO-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_U-1_SL-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_U-1_SL-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_U-1_SL-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_S-U-1_SO-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_S-U-1_SO-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "C-Proposed-U_S-U-1_SO-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+
+    // Those tests expect that <font> elements can be nested, but they don't
+    // match with the other browsers' behavior.
     "C-Proposed-FC:g_FONTc:b.sz:6-1_SI-dM": true,
     "C-Proposed-FC:g_FONTc:b.sz:6-1_SI-body": true,
     "C-Proposed-FC:g_FONTc:b.sz:6-1_SI-div": true,
-    "C-Proposed-HC:g_SPAN.ass.s:c:rgb-1_SW-dM": true,
-    "C-Proposed-HC:g_SPAN.ass.s:c:rgb-1_SW-body": true,
-    "C-Proposed-HC:g_SPAN.ass.s:c:rgb-1_SW-div": true,
-    "C-Proposed-FN:c_FONTf:a-1_SI-dM": true,
-    "C-Proposed-FN:c_FONTf:a-1_SI-body": true,
-    "C-Proposed-FN:c_FONTf:a-1_SI-div": true,
-    "C-Proposed-FN:c_FONTf:a-2_SL-dM": true,
-    "C-Proposed-FN:c_FONTf:a-2_SL-body": true,
-    "C-Proposed-FN:c_FONTf:a-2_SL-div": true,
+
     "C-Proposed-FS:1_SPAN.ass.s:fs:large-1_SW-dM": true,
     "C-Proposed-FS:1_SPAN.ass.s:fs:large-1_SW-body": true,
     "C-Proposed-FS:1_SPAN.ass.s:fs:large-1_SW-div": true,
-    "C-Proposed-FS:5_FONTsz:1.s:fs:xs-1_SW-dM": true,
-    "C-Proposed-FS:5_FONTsz:1.s:fs:xs-1_SW-body": true,
-    "C-Proposed-FS:5_FONTsz:1.s:fs:xs-1_SW-div": true,
+
+    // Those tests expect that <font> elements can be nested, but they don't
+    // match with the other browsers' behavior.
     "C-Proposed-FS:2_FONTc:b.sz:6-1_SI-dM": true,
     "C-Proposed-FS:2_FONTc:b.sz:6-1_SI-body": true,
     "C-Proposed-FS:2_FONTc:b.sz:6-1_SI-div": true,
+
     "C-Proposed-FS:larger_FONTsz:4-dM": true,
     "C-Proposed-FS:larger_FONTsz:4-body": true,
     "C-Proposed-FS:larger_FONTsz:4-div": true,
@@ -722,18 +708,18 @@ const knownFailures = {
     "C-Proposed-FB:h1_ADDRESS-FONT.ass.sz:4-1_SW-dM": true,
     "C-Proposed-FB:h1_ADDRESS-FONT.ass.sz:4-1_SW-body": true,
     "C-Proposed-FB:h1_ADDRESS-FONT.ass.sz:4-1_SW-div": true,
-    "CC-Proposed-I_I-1_SL-dM": true,
-    "CC-Proposed-I_I-1_SL-body": true,
-    "CC-Proposed-I_I-1_SL-div": true,
-    "CC-Proposed-I_B-1_SL-dM": true,
-    "CC-Proposed-I_B-1_SL-body": true,
-    "CC-Proposed-I_B-1_SL-div": true,
-    "CC-Proposed-I_B-1_SW-dM": true,
-    "CC-Proposed-I_B-1_SW-body": true,
-    "CC-Proposed-I_B-1_SW-div": true,
-    "CC-Proposed-BC:gray_SPANs:bc:b-1_SI-dM": true,
-    "CC-Proposed-BC:gray_SPANs:bc:b-1_SI-body": true,
-    "CC-Proposed-BC:gray_SPANs:bc:b-1_SI-div": true,
+    "CC-Proposed-I_I-1_SL-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_I-1_SL-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_I-1_SL-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_B-1_SL-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_B-1_SL-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_B-1_SL-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_B-1_SW-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_B-1_SW-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-I_B-1_SW-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-BC:gray_SPANs:bc:b-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-BC:gray_SPANs:bc:b-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "CC-Proposed-BC:gray_SPANs:bc:b-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "CC-Proposed-BC:gray_P-SPANs:bc:b-3_SL-dM": true,
     "CC-Proposed-BC:gray_P-SPANs:bc:b-3_SL-body": true,
     "CC-Proposed-BC:gray_P-SPANs:bc:b-3_SL-div": true,
@@ -743,12 +729,6 @@ const knownFailures = {
     "CC-Proposed-BC:gray_SPANs:bc:b-2_SR-dM": true,
     "CC-Proposed-BC:gray_SPANs:bc:b-2_SR-body": true,
     "CC-Proposed-BC:gray_SPANs:bc:b-2_SR-div": true,
-    "CC-Proposed-FN:c_FONTf:a-1_SI-dM": true,
-    "CC-Proposed-FN:c_FONTf:a-1_SI-body": true,
-    "CC-Proposed-FN:c_FONTf:a-1_SI-div": true,
-    "CC-Proposed-FN:c_FONTf:a-2_SL-dM": true,
-    "CC-Proposed-FN:c_FONTf:a-2_SL-body": true,
-    "CC-Proposed-FN:c_FONTf:a-2_SL-div": true,
     "CC-Proposed-FS:1_SPANs:fs:l-1_SW-dM": true,
     "CC-Proposed-FS:1_SPANs:fs:l-1_SW-body": true,
     "CC-Proposed-FS:1_SPANs:fs:l-1_SW-div": true,
@@ -794,9 +774,9 @@ const knownFailures = {
     "U-Proposed-I_I-P3-1_SO2-dM": true,
     "U-Proposed-I_I-P3-1_SO2-body": true,
     "U-Proposed-I_I-P3-1_SO2-div": true,
-    "U-Proposed-U_U-S-1_SO-dM": true,
-    "U-Proposed-U_U-S-1_SO-body": true,
-    "U-Proposed-U_U-S-1_SO-div": true,
+    "U-Proposed-U_U-S-1_SO-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-U_U-S-1_SO-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-U_U-S-1_SO-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "U-Proposed-U_U-S-2_SI-dM": true,
     "U-Proposed-U_U-S-2_SI-body": true,
     "U-Proposed-U_U-S-2_SI-div": true,
@@ -806,12 +786,12 @@ const knownFailures = {
     "U-Proposed-S_DEL-1_SW-dM": true,
     "U-Proposed-S_DEL-1_SW-body": true,
     "U-Proposed-S_DEL-1_SW-div": true,
-    "U-Proposed-S_S-U-1_SI-dM": true,
-    "U-Proposed-S_S-U-1_SI-body": true,
-    "U-Proposed-S_S-U-1_SI-div": true,
-    "U-Proposed-S_U-S-1_SI-dM": true,
-    "U-Proposed-S_U-S-1_SI-body": true,
-    "U-Proposed-S_U-S-1_SI-div": true,
+    "U-Proposed-S_S-U-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-S_S-U-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-S_S-U-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-S_U-S-1_SI-dM": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-S_U-S-1_SI-body": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
+    "U-Proposed-S_U-S-1_SI-div": !SpecialPowers.getBoolPref("editor.inline_style.range.compatible_with_the_other_browsers", true),
     "U-Proposed-SUB_SPANs:va:sub-1_SW-dM": true,
     "U-Proposed-SUB_SPANs:va:sub-1_SW-body": true,
     "U-Proposed-SUB_SPANs:va:sub-1_SW-div": true,
@@ -980,9 +960,6 @@ const knownFailures = {
     "I-Proposed-IHR_B-I-1_SMR-dM": true,
     "I-Proposed-IHR_B-I-1_SMR-body": true,
     "I-Proposed-IHR_B-I-1_SMR-div": true,
-    "I-Proposed-IBR_LI-1_SC-dM": true,
-    "I-Proposed-IBR_LI-1_SC-body": true,
-    "I-Proposed-IBR_LI-1_SC-div": true,
     "I-Proposed-IIMG:._SPAN-IMG-1_SO-dM": true,
     "I-Proposed-IIMG:._SPAN-IMG-1_SO-body": true,
     "I-Proposed-IIMG:._SPAN-IMG-1_SO-div": true,
