@@ -513,18 +513,12 @@ already_AddRefed<gfx::SourceSurface> CanvasTranslator::LookupExternalSurface(
 }
 
 already_AddRefed<gfx::GradientStops> CanvasTranslator::GetOrCreateGradientStops(
-    gfx::GradientStop* aRawStops, uint32_t aNumStops,
-    gfx::ExtendMode aExtendMode) {
+    gfx::DrawTarget* aDrawTarget, gfx::GradientStop* aRawStops,
+    uint32_t aNumStops, gfx::ExtendMode aExtendMode) {
+  MOZ_ASSERT(aDrawTarget);
   nsTArray<gfx::GradientStop> rawStopArray(aRawStops, aNumStops);
-  RefPtr<DrawTarget> drawTarget = GetReferenceDrawTarget();
-  if (!drawTarget) {
-    // We might end up with a null reference draw target due to a device
-    // failure, just return false so that we can recover.
-    return nullptr;
-  }
-
   return gfx::gfxGradientCache::GetOrCreateGradientStops(
-      drawTarget, rawStopArray, aExtendMode);
+      aDrawTarget, rawStopArray, aExtendMode);
 }
 
 gfx::DataSourceSurface* CanvasTranslator::LookupDataSurface(
