@@ -9,11 +9,13 @@
 #include "gfxContext.h"
 #include "gfxUtils.h"
 #include "mozilla/gfx/2D.h"
+#include "mozilla/BasePrincipal.h"
 #include "mozilla/dom/Document.h"
-#include "mozilla/dom/SVGDocument.h"
+#include "mozilla/extensions/WebExtensionPolicy.h"
 #include "mozilla/StaticPrefs_svg.h"
-#include "nsSVGPaintServerFrame.h"
-#include "SVGObserverUtils.h"
+#include "mozilla/SVGObserverUtils.h"
+#include "mozilla/SVGUtils.h"
+#include "SVGPaintServerFrame.h"
 
 using namespace mozilla::gfx;
 using namespace mozilla::image;
@@ -51,7 +53,8 @@ bool SVGContextPaint::IsAllowedForImageFromURI(nsIURI* aURI) {
   // specifically ones that are "ours". WebExtensions are moz-extension://
   // regardless if the extension is in-tree or not. Since we don't want
   // extension developers coming to rely on image context paint either, we only
-  // enable context-paint for extensions that are signed by Mozilla.
+  // enable context-paint for extensions that are owned by Mozilla
+  // (based on the extension permission "internal:svgContextPropertiesAllowed").
   //
   // We also allow this for browser UI icons that are served up from
   // Mozilla-controlled domains listed in the
@@ -66,12 +69,14 @@ bool SVGContextPaint::IsAllowedForImageFromURI(nsIURI* aURI) {
   RefPtr<BasePrincipal> principal =
       BasePrincipal::CreateContentPrincipal(aURI, OriginAttributes());
 
-  nsString addonId;
-  if (NS_SUCCEEDED(principal->GetAddonId(addonId))) {
-    if (StringEndsWith(addonId, NS_LITERAL_STRING("@mozilla.org")) ||
-        StringEndsWith(addonId, NS_LITERAL_STRING("@mozilla.com"))) {
-      return true;
-    }
+  RefPtr<extensions::WebExtensionPolicy> addonPolicy = principal->AddonPolicy();
+  if (addonPolicy) {
+    // Only allowed for extensions that have the
+    // internal:svgContextPropertiesAllowed permission (added internally from
+    // to Mozilla-owned extensions, see `isMozillaExtension` function
+    // defined in Extension.jsm for the exact criteria).
+    return addonPolicy->HasPermission(
+        nsGkAtoms::svgContextPropertiesAllowedPermission);
   }
 
   bool isInAllowList = false;
@@ -94,9 +99,10 @@ static void SetupInheritablePaint(const DrawTarget* aDrawTarget,
                                   SVGContextPaint* aOuterContextPaint,
                                   SVGContextPaintImpl::Paint& aTargetPaint,
                                   StyleSVGPaint nsStyleSVG::*aFillOrStroke,
+                                  nscolor aDefaultFallbackColor,
                                   imgDrawingParams& aImgParams) {
   const nsStyleSVG* style = aFrame->StyleSVG();
-  nsSVGPaintServerFrame* ps =
+  SVGPaintServerFrame* ps =
       SVGObserverUtils::GetAndObservePaintServer(aFrame, aFillOrStroke);
 
   if (ps) {
@@ -132,8 +138,8 @@ static void SetupInheritablePaint(const DrawTarget* aDrawTarget,
     }
   }
 
-  nscolor color =
-      nsSVGUtils::GetFallbackOrPaintColor(*aFrame->Style(), aFillOrStroke);
+  nscolor color = SVGUtils::GetFallbackOrPaintColor(
+      *aFrame->Style(), aFillOrStroke, aDefaultFallbackColor);
   aTargetPaint.SetColor(color);
 }
 
@@ -151,11 +157,11 @@ DrawMode SVGContextPaintImpl::Init(const DrawTarget* aDrawTarget,
     SetFillOpacity(0.0f);
   } else {
     float opacity =
-        nsSVGUtils::GetOpacity(style->mFillOpacity, aOuterContextPaint);
+        SVGUtils::GetOpacity(style->mFillOpacity, aOuterContextPaint);
 
     SetupInheritablePaint(aDrawTarget, aContextMatrix, aFrame, opacity,
                           aOuterContextPaint, mFillPaint, &nsStyleSVG::mFill,
-                          aImgParams);
+                          NS_RGB(0, 0, 0), aImgParams);
 
     SetFillOpacity(opacity);
 
@@ -167,11 +173,11 @@ DrawMode SVGContextPaintImpl::Init(const DrawTarget* aDrawTarget,
     SetStrokeOpacity(0.0f);
   } else {
     float opacity =
-        nsSVGUtils::GetOpacity(style->mStrokeOpacity, aOuterContextPaint);
+        SVGUtils::GetOpacity(style->mStrokeOpacity, aOuterContextPaint);
 
-    SetupInheritablePaint(aDrawTarget, aContextMatrix, aFrame, opacity,
-                          aOuterContextPaint, mStrokePaint,
-                          &nsStyleSVG::mStroke, aImgParams);
+    SetupInheritablePaint(
+        aDrawTarget, aContextMatrix, aFrame, opacity, aOuterContextPaint,
+        mStrokePaint, &nsStyleSVG::mStroke, NS_RGBA(0, 0, 0, 0), aImgParams);
 
     SetStrokeOpacity(opacity);
 
@@ -193,14 +199,10 @@ void SVGContextPaint::InitStrokeGeometry(gfxContext* aContext,
 
 SVGContextPaint* SVGContextPaint::GetContextPaint(nsIContent* aContent) {
   dom::Document* ownerDoc = aContent->OwnerDoc();
-  if (!ownerDoc->IsSVGDocument()) {
-    return nullptr;
-  }
 
-  auto* contextPaint = ownerDoc->AsSVGDocument()->GetCurrentContextPaint();
-  MOZ_ASSERT_IF(contextPaint, ownerDoc->IsBeingUsedAsImage());
+  const auto* contextPaint = ownerDoc->GetCurrentContextPaint();
 
-  // XXX The SVGContextPaint that SVGDocument keeps around is const. We could
+  // XXX The SVGContextPaint that Document keeps around is const. We could
   // and should keep that constness to the SVGContextPaint that we get here
   // (SVGImageContext is never changed after it is initialized).
   //
@@ -257,6 +259,9 @@ already_AddRefed<gfxPattern> SVGContextPaintImpl::Paint::GetPattern(
       pattern = mPaintDefinition.mPaintServerFrame->GetPaintServerPattern(
           mFrame, aDrawTarget, mContextMatrix, aFillOrStroke, aOpacity,
           aImgParams);
+      if (!pattern) {
+        return nullptr;
+      }
       {
         // m maps original-user-space to pattern space
         gfxMatrix m = pattern->GetMatrix();
@@ -287,22 +292,19 @@ already_AddRefed<gfxPattern> SVGContextPaintImpl::Paint::GetPattern(
       return nullptr;
   }
 
-  mPatternCache.Put(aOpacity, RefPtr{pattern});
+  mPatternCache.InsertOrUpdate(aOpacity, RefPtr{pattern});
   return pattern.forget();
 }
 
 AutoSetRestoreSVGContextPaint::AutoSetRestoreSVGContextPaint(
-    const SVGContextPaint& aContextPaint, dom::SVGDocument& aSVGDocument)
-    : mSVGDocument(aSVGDocument),
-      mOuterContextPaint(aSVGDocument.GetCurrentContextPaint()) {
-  MOZ_ASSERT(aSVGDocument.IsBeingUsedAsImage(),
-             "SVGContextPaint::GetContextPaint assumes this");
-
-  mSVGDocument.SetCurrentContextPaint(&aContextPaint);
+    const SVGContextPaint* aContextPaint, dom::Document* aDocument)
+    : mDocument(aDocument),
+      mOuterContextPaint(aDocument->GetCurrentContextPaint()) {
+  mDocument->SetCurrentContextPaint(aContextPaint);
 }
 
 AutoSetRestoreSVGContextPaint::~AutoSetRestoreSVGContextPaint() {
-  mSVGDocument.SetCurrentContextPaint(mOuterContextPaint);
+  mDocument->SetCurrentContextPaint(mOuterContextPaint);
 }
 
 // SVGEmbeddingContextPaint
