@@ -21,7 +21,7 @@
 #include "nsTHashtable.h"
 #include "prio.h"
 
-//#define DEBUG_HANDLES 1
+// #define DEBUG_HANDLES 1
 
 class nsIFile;
 class nsITimer;
@@ -122,7 +122,10 @@ class CacheFileHandle final : public nsISupports {
   PinningStatus mPinning;
 
   nsCOMPtr<nsIFile> mFile;
-  int64_t mFileSize;
+
+  // file size is atomic because it is used on main thread by
+  // nsHttpChannel::ReportNetVSCacheTelemetry()
+  Atomic<int64_t, Relaxed> mFileSize;
   PRFileDesc* mFD;  // if null then the file doesn't exists on the disk
   nsCString mKey;
 };
@@ -133,10 +136,10 @@ class CacheFileHandles {
   ~CacheFileHandles();
 
   nsresult GetHandle(const SHA1Sum::Hash* aHash, CacheFileHandle** _retval);
-  nsresult NewHandle(const SHA1Sum::Hash* aHash, bool aPriority,
-                     CacheFileHandle::PinningStatus aPinning,
-                     CacheFileHandle** _retval);
-  void RemoveHandle(CacheFileHandle* aHandlle);
+  already_AddRefed<CacheFileHandle> NewHandle(const SHA1Sum::Hash*,
+                                              bool aPriority,
+                                              CacheFileHandle::PinningStatus);
+  void RemoveHandle(CacheFileHandle* aHandle);
   void GetAllHandles(nsTArray<RefPtr<CacheFileHandle> >* _retval);
   void GetActiveHandles(nsTArray<RefPtr<CacheFileHandle> >* _retval);
   void ClearAll();
@@ -152,8 +155,8 @@ class CacheFileHandles {
 
   class HandleHashKey : public PLDHashEntryHdr {
    public:
-    typedef const SHA1Sum::Hash& KeyType;
-    typedef const SHA1Sum::Hash* KeyTypePointer;
+    using KeyType = const SHA1Sum::Hash&;
+    using KeyTypePointer = const SHA1Sum::Hash*;
 
     explicit HandleHashKey(KeyTypePointer aKey) {
       MOZ_COUNT_CTOR(HandleHashKey);
@@ -163,7 +166,7 @@ class CacheFileHandles {
     HandleHashKey(const HandleHashKey& aOther) {
       MOZ_ASSERT_UNREACHABLE("HandleHashKey copy constructor is forbidden!");
     }
-    ~HandleHashKey() { MOZ_COUNT_DTOR(HandleHashKey); }
+    MOZ_COUNTED_DTOR(HandleHashKey)
 
     bool KeyEquals(KeyTypePointer aKey) const {
       return memcmp(mHash.get(), aKey, sizeof(SHA1Sum::Hash)) == 0;
@@ -263,6 +266,7 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   static nsresult Init();
   static nsresult Shutdown();
   static nsresult OnProfile();
+  static nsresult OnDelayedStartupFinished();
   static already_AddRefed<nsIEventTarget> IOTarget();
   static already_AddRefed<CacheIOThread> IOThread();
   static bool IsOnIOThread();
@@ -282,9 +286,17 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
                            CacheFileIOListener* aCallback);
   static nsresult Read(CacheFileHandle* aHandle, int64_t aOffset, char* aBuf,
                        int32_t aCount, CacheFileIOListener* aCallback);
+  // This function must be called with a callback. The caller is responsible for
+  // releasing |aBuf|.
   static nsresult Write(CacheFileHandle* aHandle, int64_t aOffset,
                         const char* aBuf, int32_t aCount, bool aValidate,
                         bool aTruncate, CacheFileIOListener* aCallback);
+  // Similar to the above, but without the callback. Note that |aBuf| will be
+  // released by CacheFileIOManager.
+  static nsresult WriteWithoutCallback(CacheFileHandle* aHandle,
+                                       int64_t aOffset, char* aBuf,
+                                       int32_t aCount, bool aValidate,
+                                       bool aTruncate);
   // PinningDoomRestriction:
   // NO_RESTRICTION
   //    no restriction is checked, the file is simply always doomed
@@ -313,7 +325,8 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   static nsresult EvictIfOverLimit();
   static nsresult EvictAll();
   static nsresult EvictByContext(nsILoadContextInfo* aLoadContextInfo,
-                                 bool aPinning, const nsAString& aOrigin);
+                                 bool aPinned, const nsAString& aOrigin,
+                                 const nsAString& aBaseDomain = u""_ns);
 
   static nsresult InitIndexEntry(CacheFileHandle* aHandle,
                                  OriginAttrsHash aOriginAttrsHash,
@@ -323,9 +336,7 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
                                    const bool* aHasAltData,
                                    const uint16_t* aOnStartTime,
                                    const uint16_t* aOnStopTime,
-                                   const uint8_t* aContentType,
-                                   const uint16_t* aBaseDomainAccessCount,
-                                   const uint32_t aTelemetryReportID);
+                                   const uint8_t* aContentType);
 
   static nsresult UpdateIndexEntry();
 
@@ -369,13 +380,13 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   virtual ~CacheFileIOManager();
 
   nsresult InitInternal();
-  nsresult ShutdownInternal();
+  void ShutdownInternal();
 
   nsresult OpenFileInternal(const SHA1Sum::Hash* aHash, const nsACString& aKey,
                             uint32_t aFlags, CacheFileHandle** _retval);
   nsresult OpenSpecialFileInternal(const nsACString& aKey, uint32_t aFlags,
                                    CacheFileHandle** _retval);
-  nsresult CloseHandleInternal(CacheFileHandle* aHandle);
+  void CloseHandleInternal(CacheFileHandle* aHandle);
   nsresult ReadInternal(CacheFileHandle* aHandle, int64_t aOffset, char* aBuf,
                         int32_t aCount);
   nsresult WriteInternal(CacheFileHandle* aHandle, int64_t aOffset,
@@ -383,7 +394,7 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
                          bool aTruncate);
   nsresult DoomFileInternal(
       CacheFileHandle* aHandle,
-      PinningDoomRestriction aPinningStatusRestriction = NO_RESTRICTION);
+      PinningDoomRestriction aPinningDoomRestriction = NO_RESTRICTION);
   nsresult DoomFileByKeyInternal(const SHA1Sum::Hash* aHash);
   nsresult MaybeReleaseNSPRHandleInternal(CacheFileHandle* aHandle,
                                           bool aIgnoreShutdownLag = false);
@@ -395,7 +406,8 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   nsresult OverLimitEvictionInternal();
   nsresult EvictAllInternal();
   nsresult EvictByContextInternal(nsILoadContextInfo* aLoadContextInfo,
-                                  bool aPinning, const nsAString& aOrigin);
+                                  bool aPinned, const nsAString& aOrigin,
+                                  const nsAString& aBaseDomain = u""_ns);
 
   nsresult TrashDirectory(nsIFile* aFile);
   static void OnTrashTimer(nsITimer* aTimer, void* aClosure);
@@ -421,11 +433,19 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   void SyncRemoveAllCacheFiles();
 
   nsresult ScheduleMetadataWriteInternal(CacheFile* aFile);
-  nsresult UnscheduleMetadataWriteInternal(CacheFile* aFile);
-  nsresult ShutdownMetadataWriteSchedulingInternal();
+  void UnscheduleMetadataWriteInternal(CacheFile* aFile);
+  void ShutdownMetadataWriteSchedulingInternal();
 
   static nsresult CacheIndexStateChanged();
-  nsresult CacheIndexStateChangedInternal();
+  void CacheIndexStateChangedInternal();
+
+  // Dispatches a purgeHTTP background task to delete the cache directoy
+  // indicated by aCacheDirName.
+  // When this feature is enabled, a task will be dispatched at shutdown
+  // or after browser startup (to cleanup potential left-over directories)
+  nsresult DispatchPurgeTask(const nsCString& aCacheDirName,
+                             const nsCString& aSecondsToWait,
+                             const nsCString& aPurgeExtension);
 
   // Smart size calculation. UpdateSmartCacheSize() must be called on IO thread.
   // It is called in EvictIfOverLimitInternal() just before we decide whether to
@@ -441,7 +461,7 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   TimeStamp mStartTime;
   // Set true on the IO thread, CLOSE level as part of the internal shutdown
   // procedure.
-  bool mShuttingDown;
+  bool mShuttingDown{false};
   RefPtr<CacheIOThread> mIOThread;
   nsCOMPtr<nsIFile> mCacheDirectory;
 #if defined(MOZ_WIDGET_ANDROID)
@@ -451,19 +471,19 @@ class CacheFileIOManager final : public nsITimerCallback, public nsINamed {
   // w/o the profile name in the path.  Here it is stored.
   nsCOMPtr<nsIFile> mCacheProfilelessDirectory;
 #endif
-  bool mTreeCreated;
-  bool mTreeCreationFailed;
+  bool mTreeCreated{false};
+  bool mTreeCreationFailed{false};
   CacheFileHandles mHandles;
   nsTArray<CacheFileHandle*> mHandlesByLastUsed;
   nsTArray<CacheFileHandle*> mSpecialHandles;
   nsTArray<RefPtr<CacheFile> > mScheduledMetadataWrites;
   nsCOMPtr<nsITimer> mMetadataWritesTimer;
-  bool mOverLimitEvicting;
+  bool mOverLimitEvicting{false};
   // When overlimit eviction is too slow and cache size reaches 105% of the
   // limit, this flag is set and no other content is cached to prevent
   // uncontrolled cache growing.
-  bool mCacheSizeOnHardLimit;
-  bool mRemovingTrashDirs;
+  bool mCacheSizeOnHardLimit{false};
+  bool mRemovingTrashDirs{false};
   nsCOMPtr<nsITimer> mTrashTimer;
   nsCOMPtr<nsIFile> mTrashDir;
   nsCOMPtr<nsIDirectoryEnumerator> mTrashDirEnumerator;
