@@ -4,6 +4,8 @@
 
 #include "CacheIOThread.h"
 #include "CacheFileIOManager.h"
+#include "CacheLog.h"
+#include "CacheObserver.h"
 
 #include "nsIRunnable.h"
 #include "nsISupportsImpl.h"
@@ -13,26 +15,22 @@
 #include "nsThreadUtils.h"
 #include "mozilla/EventQueue.h"
 #include "mozilla/IOInterposer.h"
+#include "mozilla/ProfilerLabels.h"
 #include "mozilla/ThreadEventQueue.h"
-#include "GeckoProfiler.h"
+#include "mozilla/Telemetry.h"
+#include "mozilla/TelemetryHistogramEnums.h"
 
 #ifdef XP_WIN
 #  include <windows.h>
 #endif
 
-#ifdef MOZ_TASK_TRACER
-#  include "GeckoTaskTracer.h"
-#  include "TracedTaskCommon.h"
-#endif
-
-namespace mozilla {
-namespace net {
+namespace mozilla::net {
 
 namespace {  // anon
 
 class CacheIOTelemetry {
  public:
-  typedef CacheIOThread::EventQueue::size_type size_type;
+  using size_type = CacheIOThread::EventQueue::size_type;
   static size_type mMinLengthToReport[CacheIOThread::LAST_LEVEL];
   static void Report(uint32_t aLevel, size_type aLength);
 };
@@ -83,80 +81,43 @@ namespace detail {
  * shutdown to prevent any IO shutdown hangs.
  * Mainly designed for using Win32 CancelSynchronousIo function.
  */
-class BlockingIOWatcher {
+class NativeThreadHandle {
 #ifdef XP_WIN
-  typedef BOOL(WINAPI* TCancelSynchronousIo)(HANDLE hThread);
-  TCancelSynchronousIo mCancelSynchronousIo;
   // The native handle to the thread
   HANDLE mThread;
-  // Event signaling back to the main thread, see NotifyOperationDone.
-  HANDLE mEvent;
 #endif
 
  public:
   // Created and destroyed on the main thread only
-  BlockingIOWatcher();
-  ~BlockingIOWatcher();
+  NativeThreadHandle();
+  ~NativeThreadHandle();
 
   // Called on the IO thread to grab the platform specific
   // reference to it.
   void InitThread();
   // If there is a blocking operation being handled on the IO
   // thread, this is called on the main thread during shutdown.
-  // Waits for notification from the IO thread for up to two seconds.
-  // If that times out, it attempts to cancel the IO operation.
-  void WatchAndCancel(Monitor& aMonitor);
-  // Called by the IO thread after each operation has been
-  // finished (after each Run() call).  This wakes the main
-  // thread up and makes WatchAndCancel() early exit and become
-  // a no-op.
-  void NotifyOperationDone();
+  void CancelBlockingIO(Monitor& aMonitor);
 };
 
 #ifdef XP_WIN
 
-BlockingIOWatcher::BlockingIOWatcher()
-    : mCancelSynchronousIo(NULL), mThread(NULL), mEvent(NULL) {
-  HMODULE kernel32_dll = GetModuleHandle("kernel32.dll");
-  if (!kernel32_dll) {
-    return;
-  }
+NativeThreadHandle::NativeThreadHandle() : mThread(NULL) {}
 
-  FARPROC ptr = GetProcAddress(kernel32_dll, "CancelSynchronousIo");
-  if (!ptr) {
-    return;
-  }
-
-  mCancelSynchronousIo = reinterpret_cast<TCancelSynchronousIo>(ptr);
-
-  mEvent = ::CreateEventW(NULL, TRUE, FALSE, NULL);
-}
-
-BlockingIOWatcher::~BlockingIOWatcher() {
-  if (mEvent) {
-    CloseHandle(mEvent);
-  }
+NativeThreadHandle::~NativeThreadHandle() {
   if (mThread) {
     CloseHandle(mThread);
   }
 }
 
-void BlockingIOWatcher::InitThread() {
+void NativeThreadHandle::InitThread() {
   // GetCurrentThread() only returns a pseudo handle, hence DuplicateHandle
   ::DuplicateHandle(GetCurrentProcess(), GetCurrentThread(),
                     GetCurrentProcess(), &mThread, 0, FALSE,
                     DUPLICATE_SAME_ACCESS);
 }
 
-void BlockingIOWatcher::WatchAndCancel(Monitor& aMonitor) {
-  if (!mEvent) {
-    return;
-  }
-
-  // Reset before we enter the monitor to raise the chance we catch
-  // the currently pending IO op completion.
-  ::ResetEvent(mEvent);
-
+void NativeThreadHandle::CancelBlockingIO(Monitor& aMonitor) {
   HANDLE thread;
   {
     MonitorAutoLock lock(aMonitor);
@@ -167,31 +128,13 @@ void BlockingIOWatcher::WatchAndCancel(Monitor& aMonitor) {
     }
   }
 
-  LOG(("Blocking IO operation pending on IO thread, waiting..."));
-
-  // It seems wise to use the I/O lag time as a maximum time to wait
-  // for an operation to finish.  When that times out and cancelation
-  // succeeds, there will be no other IO operation permitted.  By default
-  // this is two seconds.
-  uint32_t maxLag =
-      std::min<uint32_t>(5, CacheObserver::MaxShutdownIOLag()) * 1000;
-
-  DWORD result = ::WaitForSingleObject(mEvent, maxLag);
-  if (result == WAIT_TIMEOUT) {
-    LOG(("CacheIOThread: Attempting to cancel a long blocking IO operation"));
-    BOOL result = mCancelSynchronousIo(thread);
-    if (result) {
-      LOG(("  cancelation signal succeeded"));
-    } else {
-      DWORD error = GetLastError();
-      LOG(("  cancelation signal failed with GetLastError=%u", error));
-    }
-  }
-}
-
-void BlockingIOWatcher::NotifyOperationDone() {
-  if (mEvent) {
-    ::SetEvent(mEvent);
+  LOG(("CacheIOThread: Attempting to cancel a long blocking IO operation"));
+  BOOL result = ::CancelSynchronousIo(thread);
+  if (result) {
+    LOG(("  cancelation signal succeeded"));
+  } else {
+    DWORD error = GetLastError();
+    LOG(("  cancelation signal failed with GetLastError=%lu", error));
   }
 }
 
@@ -199,11 +142,10 @@ void BlockingIOWatcher::NotifyOperationDone() {
 
 // Stub code only (we don't implement IO cancelation for this platform)
 
-BlockingIOWatcher::BlockingIOWatcher() = default;
-BlockingIOWatcher::~BlockingIOWatcher() = default;
-void BlockingIOWatcher::InitThread() {}
-void BlockingIOWatcher::WatchAndCancel(Monitor&) {}
-void BlockingIOWatcher::NotifyOperationDone() {}
+NativeThreadHandle::NativeThreadHandle() = default;
+NativeThreadHandle::~NativeThreadHandle() = default;
+void NativeThreadHandle::InitThread() {}
+void NativeThreadHandle::CancelBlockingIO(Monitor&) {}
 
 #endif
 
@@ -213,22 +155,7 @@ CacheIOThread* CacheIOThread::sSelf = nullptr;
 
 NS_IMPL_ISUPPORTS(CacheIOThread, nsIThreadObserver)
 
-CacheIOThread::CacheIOThread()
-    : mMonitor("CacheIOThread"),
-      mThread(nullptr),
-      mXPCOMThread(nullptr),
-      mLowestLevelWaiting(LAST_LEVEL),
-      mCurrentlyExecutingLevel(0),
-      mHasXPCOMEvents(false),
-      mRerunCurrentEvent(false),
-      mShutdown(false),
-      mIOCancelableEvents(0),
-      mEventCounter(0)
-#ifdef DEBUG
-      ,
-      mInsideLoop(true)
-#endif
-{
+CacheIOThread::CacheIOThread() {
   for (auto& item : mQueueLength) {
     item = 0;
   }
@@ -255,9 +182,13 @@ nsresult CacheIOThread::Init() {
     MonitorAutoLock lock(mMonitor);
     // Yeah, there is not a thread yet, but we want to make sure
     // the sequencing is correct.
-    mBlockingIOWatcher = MakeUnique<detail::BlockingIOWatcher>();
+    mNativeThreadHandle = MakeUnique<detail::NativeThreadHandle>();
   }
 
+  // Increase the reference count while spawning a new thread.
+  // If PR_CreateThread succeeds, we will forget this reference and the thread
+  // will be responsible to release it when it completes.
+  RefPtr<CacheIOThread> self = this;
   mThread =
       PR_CreateThread(PR_USER_THREAD, ThreadFunc, this, PR_PRIORITY_NORMAL,
                       PR_GLOBAL_THREAD, PR_JOINABLE_THREAD, 128 * 1024);
@@ -265,6 +196,11 @@ nsresult CacheIOThread::Init() {
     return NS_ERROR_FAILURE;
   }
 
+  // IMPORTANT: The thread now owns this reference, so it's important that we
+  // leak it here, otherwise we'll end up with a bad refcount.
+  // See the dont_AddRef in ThreadFunc().
+  Unused << self.forget().take();
+
   return NS_OK;
 }
 
@@ -283,8 +219,9 @@ nsresult CacheIOThread::Dispatch(already_AddRefed<nsIRunnable> aRunnable,
 
   MonitorAutoLock lock(mMonitor);
 
-  if (mShutdown && (PR_GetCurrentThread() != mThread))
+  if (mShutdown && (PR_GetCurrentThread() != mThread)) {
     return NS_ERROR_UNEXPECTED;
+  }
 
   return DispatchInternal(runnable.forget(), aLevel);
 }
@@ -295,8 +232,9 @@ nsresult CacheIOThread::DispatchAfterPendingOpens(nsIRunnable* aRunnable) {
 
   MonitorAutoLock lock(mMonitor);
 
-  if (mShutdown && (PR_GetCurrentThread() != mThread))
+  if (mShutdown && (PR_GetCurrentThread() != mThread)) {
     return NS_ERROR_UNEXPECTED;
+  }
 
   // Move everything from later executed OPEN level to the OPEN_PRIORITY level
   // where we post the (eviction) runnable.
@@ -311,12 +249,8 @@ nsresult CacheIOThread::DispatchAfterPendingOpens(nsIRunnable* aRunnable) {
 nsresult CacheIOThread::DispatchInternal(
     already_AddRefed<nsIRunnable> aRunnable, uint32_t aLevel) {
   nsCOMPtr<nsIRunnable> runnable(aRunnable);
-#ifdef MOZ_TASK_TRACER
-  if (tasktracer::IsStartLogging()) {
-    runnable = tasktracer::CreateTracedRunnable(runnable.forget());
-    (static_cast<tasktracer::TracedRunnable*>(runnable.get()))->DispatchTask();
-  }
-#endif
+
+  LogRunnable::LogDispatch(runnable.get());
 
   if (NS_WARN_IF(!runnable)) return NS_ERROR_NULL_POINTER;
 
@@ -383,7 +317,7 @@ void CacheIOThread::Shutdown() {
 void CacheIOThread::CancelBlockingIO() {
   // This is an attempt to cancel any blocking I/O operation taking
   // too long time.
-  if (!mBlockingIOWatcher) {
+  if (!mNativeThreadHandle) {
     return;
   }
 
@@ -394,7 +328,7 @@ void CacheIOThread::CancelBlockingIO() {
 
   // OK, when we are here, we are processing an IO on the thread that
   // can be cancelled.
-  mBlockingIOWatcher->WatchAndCancel(mMonitor);
+  mNativeThreadHandle->CancelBlockingIO(mMonitor);
 }
 
 already_AddRefed<nsIEventTarget> CacheIOThread::Target() {
@@ -420,7 +354,9 @@ void CacheIOThread::ThreadFunc(void* aClosure) {
   NS_SetCurrentThreadName("Cache2 I/O");
 
   mozilla::IOInterposer::RegisterCurrentThread();
-  CacheIOThread* thread = static_cast<CacheIOThread*>(aClosure);
+  // We hold on to this reference for the duration of the thread.
+  RefPtr<CacheIOThread> thread =
+      dont_AddRef(static_cast<CacheIOThread*>(aClosure));
   thread->ThreadFunc();
   mozilla::IOInterposer::UnregisterCurrentThread();
 }
@@ -431,11 +367,11 @@ void CacheIOThread::ThreadFunc() {
   {
     MonitorAutoLock lock(mMonitor);
 
-    MOZ_ASSERT(mBlockingIOWatcher);
-    mBlockingIOWatcher->InitThread();
+    MOZ_ASSERT(mNativeThreadHandle);
+    mNativeThreadHandle->InitThread();
 
-    auto queue = MakeRefPtr<ThreadEventQueue<mozilla::EventQueue>>(
-        MakeUnique<mozilla::EventQueue>());
+    auto queue =
+        MakeRefPtr<ThreadEventQueue>(MakeUnique<mozilla::EventQueue>());
     nsCOMPtr<nsIThread> xpcomThread =
         nsThreadManager::get().CreateCurrentThread(queue,
                                                    nsThread::NOT_MAIN_THREAD);
@@ -444,6 +380,7 @@ void CacheIOThread::ThreadFunc() {
     if (threadInternal) threadInternal->SetObserver(this);
 
     mXPCOMThread = xpcomThread.forget().take();
+    nsCOMPtr<nsIThread> thread = NS_GetCurrentThread();
 
     lock.NotifyAll();
 
@@ -464,12 +401,10 @@ void CacheIOThread::ThreadFunc() {
         bool processedEvent;
         nsresult rv;
         do {
-          nsIThread* thread = mXPCOMThread;
           rv = thread->ProcessNextEvent(false, &processedEvent);
 
           ++mEventCounter;
-          MOZ_ASSERT(mBlockingIOWatcher);
-          mBlockingIOWatcher->NotifyOperationDone();
+          MOZ_ASSERT(mNativeThreadHandle);
         } while (NS_SUCCEEDED(rv) && processedEvent);
       }
 
@@ -495,7 +430,6 @@ void CacheIOThread::ThreadFunc() {
       }
 
       AUTO_PROFILER_LABEL("CacheIOThread::ThreadFunc::Wait", IDLE);
-      AUTO_PROFILER_THREAD_SLEEP;
       lock.Wait();
 
     } while (true);
@@ -512,8 +446,8 @@ void CacheIOThread::ThreadFunc() {
 }
 
 void CacheIOThread::LoopOneLevel(uint32_t aLevel) {
-  EventQueue events;
-  events.SwapElements(mEventQueue[aLevel]);
+  mMonitor.AssertCurrentThreadOwns();
+  EventQueue events = std::move(mEventQueue[aLevel]);
   EventQueue::size_type length = events.Length();
 
   mCurrentlyExecutingLevel = aLevel;
@@ -542,14 +476,16 @@ void CacheIOThread::LoopOneLevel(uint32_t aLevel) {
       // this flag.
       mRerunCurrentEvent = false;
 
+      LogRunnable::Run log(events[index].get());
+
       events[index]->Run();
 
-      MOZ_ASSERT(mBlockingIOWatcher);
-      mBlockingIOWatcher->NotifyOperationDone();
+      MOZ_ASSERT(mNativeThreadHandle);
 
       if (mRerunCurrentEvent) {
         // The event handler yields to higher priority events and wants to
         // rerun.
+        log.WillRunAgain();
         returnEvents = true;
         break;
       }
@@ -572,11 +508,11 @@ void CacheIOThread::LoopOneLevel(uint32_t aLevel) {
     events.RemoveElementsAt(0, index);
     // Move events that might have been scheduled on this queue to the tail to
     // preserve the expected per-queue FIFO order.
-    if (!events.AppendElements(std::move(mEventQueue[aLevel]))) {
-      MOZ_CRASH("Can't allocate memory for cache IO thread queue");
-    }
+    // XXX(Bug 1631371) Check if this should use a fallible operation as it
+    // pretended earlier.
+    events.AppendElements(std::move(mEventQueue[aLevel]));
     // And finally move everything back to the main queue.
-    events.SwapElements(mEventQueue[aLevel]);
+    mEventQueue[aLevel] = std::move(events);
   }
 }
 
@@ -644,5 +580,4 @@ CacheIOThread::Cancelable::~Cancelable() {
   }
 }
 
-}  // namespace net
-}  // namespace mozilla
+}  // namespace mozilla::net
