@@ -11,6 +11,7 @@
 #include "nsHttp.h"
 #include "nsHttpAuthCache.h"
 #include "nsHttpConnectionMgr.h"
+#include "AlternateServices.h"
 #include "ASpdySession.h"
 #include "HttpTrafficAnalyzer.h"
 
@@ -24,8 +25,21 @@
 #include "nsIHttpProtocolHandler.h"
 #include "nsIObserver.h"
 #include "nsISpeculativeConnect.h"
+#include "nsTHashMap.h"
+#include "nsTHashSet.h"
 
-class nsIHttpChannel;
+#ifdef DEBUG
+#  include "nsIOService.h"
+#endif
+
+// XXX These includes can be replaced by forward declarations by moving the On*
+// method implementations to the cpp file
+#include "nsIChannel.h"
+#include "nsIHttpChannel.h"
+#include "nsSocketTransportService2.h"
+
+class nsIHttpActivityDistributor;
+class nsIHttpUpgradeListener;
 class nsIPrefBranch;
 class nsICancelable;
 class nsICookieService;
@@ -34,18 +48,20 @@ class nsIRequestContextService;
 class nsISiteSecurityService;
 class nsIStreamConverterService;
 
-namespace mozilla {
-namespace net {
-
-bool OnSocketThread();
+namespace mozilla::net {
 
 class ATokenBucketEvent;
 class EventTokenBucket;
 class Tickler;
 class nsHttpConnection;
 class nsHttpConnectionInfo;
-class nsHttpTransaction;
+class HttpBaseChannel;
+class HttpHandlerInitArgs;
+class HttpTransactionShell;
 class AltSvcMapping;
+class DNSUtils;
+class TRRServiceChannel;
+class SocketProcessChild;
 
 /*
  * FRAMECHECK_LAX - no check
@@ -75,8 +91,12 @@ class AltSvcMapping;
  * FRAMECHECK_STRICT - we also do not allow case 2) and 3) from
  *                     FRAMECHECK_BARELY.
  */
-enum FrameCheckLevel { FRAMECHECK_LAX, FRAMECHECK_BARELY,
-                       FRAMECHECK_STRICT_CHUNKED, FRAMECHECK_STRICT };
+enum FrameCheckLevel {
+  FRAMECHECK_LAX,
+  FRAMECHECK_BARELY,
+  FRAMECHECK_STRICT_CHUNKED,
+  FRAMECHECK_STRICT
+};
 
 //-----------------------------------------------------------------------------
 // nsHttpHandler - protocol handler for HTTP and HTTPS
@@ -96,14 +116,14 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
 
   static already_AddRefed<nsHttpHandler> GetInstance();
 
-  MOZ_MUST_USE nsresult
-  AddStandardRequestHeaders(nsHttpRequestHead*, bool isSecure,
-                            nsContentPolicyType aContentPolicyType);
-  MOZ_MUST_USE nsresult AddConnectionHeader(nsHttpRequestHead*,
-                                            uint32_t capabilities);
+  [[nodiscard]] nsresult AddStandardRequestHeaders(
+      nsHttpRequestHead*, bool isSecure,
+      ExtContentPolicyType aContentPolicyType,
+      bool aShouldResistFingerprinting);
+  [[nodiscard]] nsresult AddConnectionHeader(nsHttpRequestHead*, uint32_t caps);
   bool IsAcceptableEncoding(const char* encoding, bool isSecure);
 
-  const nsCString& UserAgent();
+  const nsCString& UserAgent(bool aShouldResistFingerprinting);
 
   enum HttpVersion HttpVersion() { return mHttpVersion; }
   enum HttpVersion ProxyHttpVersion() { return mProxyHttpVersion; }
@@ -116,9 +136,7 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   PRIntervalTime ResponseTimeoutEnabled() { return mResponseTimeoutEnabled; }
   uint32_t NetworkChangedTimeout() { return mNetworkChangedTimeout; }
   uint16_t MaxRequestAttempts() { return mMaxRequestAttempts; }
-  const char* DefaultSocketType() {
-    return mDefaultSocketType.IsVoid() ? nullptr : mDefaultSocketType.get();
-  }
+  const nsCString& DefaultSocketType() { return mDefaultSocketType; }
   uint32_t PhishyUserPassLength() { return mPhishyUserPassLength; }
   uint8_t GetQoSBits() { return mQoSBits; }
   uint16_t GetIdleSynTimeout() { return mIdleSynTimeout; }
@@ -131,11 +149,6 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
     return mEnablePersistentHttpsCaching;
   }
 
-  bool IsSpdyEnabled() { return mEnableSpdy; }
-  bool IsHttp2Enabled() { return mHttp2Enabled; }
-  bool EnforceHttp2TlsProfile() { return mEnforceHttp2TlsProfile; }
-  bool CoalesceSpdy() { return mCoalesceSpdy; }
-  bool UseSpdyPersistentSettings() { return mSpdyPersistentSettings; }
   uint32_t SpdySendingChunkSize() { return mSpdySendingChunkSize; }
   uint32_t SpdySendBufferSize() { return mSpdySendBufferSize; }
   uint32_t SpdyPushAllowance() { return mSpdyPushAllowance; }
@@ -143,7 +156,6 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   uint32_t DefaultSpdyConcurrent() { return mDefaultSpdyConcurrent; }
   PRIntervalTime SpdyPingThreshold() { return mSpdyPingThreshold; }
   PRIntervalTime SpdyPingTimeout() { return mSpdyPingTimeout; }
-  bool AllowPush() { return mAllowPush; }
   bool AllowAltSvc() { return mEnableAltSvc; }
   bool AllowAltSvcOE() { return mEnableAltSvcOE; }
   bool AllowOriginExtension() { return mEnableOriginExtension; }
@@ -155,8 +167,6 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   bool CriticalRequestPrioritization() {
     return mCriticalRequestPrioritization;
   }
-  bool UseH2Deps() { return mUseH2Deps; }
-  bool IsH2WebsocketsEnabled() { return mEnableH2Websockets; }
 
   uint32_t MaxConnectionsPerOrigin() {
     return mMaxPersistentConnectionsPerServer;
@@ -210,29 +220,6 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
     return mTCPKeepaliveLongLivedIdleTimeS;
   }
 
-  bool UseFastOpen() {
-    return mUseFastOpen && mFastOpenSupported &&
-           (mFastOpenStallsCounter < mFastOpenStallsLimit) &&
-           (mFastOpenConsecutiveFailureCounter <
-            mFastOpenConsecutiveFailureLimit);
-  }
-  // If one of tcp connections return PR_NOT_TCP_SOCKET_ERROR while trying
-  // fast open, it means that Fast Open is turned off so we will not try again
-  // until a restart. This is only on Linux.
-  void SetFastOpenNotSupported() { mFastOpenSupported = false; }
-
-  void IncrementFastOpenConsecutiveFailureCounter();
-
-  void ResetFastOpenConsecutiveFailureCounter() {
-    mFastOpenConsecutiveFailureCounter = 0;
-  }
-
-  void IncrementFastOpenStallsCounter();
-  uint32_t CheckIfConnectionIsStalledOnlyIfIdleForThisAmountOfSeconds() {
-    return mFastOpenStallsIdleTime;
-  }
-  uint32_t FastOpenStallsTimeout() { return mFastOpenStallsTimeout; }
-
   // returns the HTTP framing check level preference, as controlled with
   // network.http.enforce-framing.http1 and network.http.enforce-framing.soft
   FrameCheckLevel GetEnforceH1Framing() { return mEnforceH1Framing; }
@@ -240,7 +227,17 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   nsHttpAuthCache* AuthCache(bool aPrivate) {
     return aPrivate ? &mPrivateAuthCache : &mAuthCache;
   }
-  nsHttpConnectionMgr* ConnMgr() { return mConnMgr; }
+  nsHttpConnectionMgr* ConnMgr() {
+    MOZ_ASSERT_IF(nsIOService::UseSocketProcess(), XRE_IsSocketProcess());
+    return mConnMgr->AsHttpConnectionMgr();
+  }
+
+  AltSvcCache* AltServiceCache() const {
+    MOZ_ASSERT(XRE_IsParentProcess());
+    return mAltSvcCache.get();
+  }
+
+  void ClearHostMapping(nsHttpConnectionInfo* aConnInfo);
 
   // cache support
   uint32_t GenerateUniqueID() { return ++mLastUniqueID; }
@@ -259,93 +256,97 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   // Called to kick-off a new transaction, by default the transaction
   // will be put on the pending transaction queue if it cannot be
   // initiated at this time.  Callable from any thread.
-  MOZ_MUST_USE nsresult InitiateTransaction(nsHttpTransaction* trans,
-                                            int32_t priority) {
-    return mConnMgr->AddTransaction(trans, priority);
-  }
+  [[nodiscard]] nsresult InitiateTransaction(HttpTransactionShell* trans,
+                                             int32_t priority);
 
   // This function is also called to kick-off a new transaction. But the new
   // transaction will take a sticky connection from |transWithStickyConn|
   // and reuse it.
-  MOZ_MUST_USE nsresult
-  InitiateTransactionWithStickyConn(nsHttpTransaction* trans, int32_t priority,
-                                    nsHttpTransaction* transWithStickyConn) {
-    return mConnMgr->AddTransactionWithStickyConn(trans, priority,
-                                                  transWithStickyConn);
-  }
+  [[nodiscard]] nsresult InitiateTransactionWithStickyConn(
+      HttpTransactionShell* trans, int32_t priority,
+      HttpTransactionShell* transWithStickyConn);
 
   // Called to change the priority of an existing transaction that has
   // already been initiated.
-  MOZ_MUST_USE nsresult RescheduleTransaction(nsHttpTransaction* trans,
-                                              int32_t priority) {
-    return mConnMgr->RescheduleTransaction(trans, priority);
-  }
+  [[nodiscard]] nsresult RescheduleTransaction(HttpTransactionShell* trans,
+                                               int32_t priority);
 
-  void UpdateClassOfServiceOnTransaction(nsHttpTransaction* trans,
-                                         uint32_t classOfService) {
-    mConnMgr->UpdateClassOfServiceOnTransaction(trans, classOfService);
-  }
+  void UpdateClassOfServiceOnTransaction(HttpTransactionShell* trans,
+                                         const ClassOfService& classOfService);
 
   // Called to cancel a transaction, which may or may not be assigned to
   // a connection.  Callable from any thread.
-  MOZ_MUST_USE nsresult CancelTransaction(nsHttpTransaction* trans,
-                                          nsresult reason) {
-    return mConnMgr->CancelTransaction(trans, reason);
-  }
+  [[nodiscard]] nsresult CancelTransaction(HttpTransactionShell* trans,
+                                           nsresult reason);
 
   // Called when a connection is done processing a transaction.  Callable
   // from any thread.
-  MOZ_MUST_USE nsresult ReclaimConnection(nsHttpConnection* conn) {
+  [[nodiscard]] nsresult ReclaimConnection(HttpConnectionBase* conn) {
     return mConnMgr->ReclaimConnection(conn);
   }
 
-  MOZ_MUST_USE nsresult ProcessPendingQ(nsHttpConnectionInfo* cinfo) {
+  [[nodiscard]] nsresult ProcessPendingQ(nsHttpConnectionInfo* cinfo) {
     return mConnMgr->ProcessPendingQ(cinfo);
   }
 
-  MOZ_MUST_USE nsresult ProcessPendingQ() {
+  [[nodiscard]] nsresult ProcessPendingQ() {
     return mConnMgr->ProcessPendingQ();
   }
 
-  MOZ_MUST_USE nsresult GetSocketThreadTarget(nsIEventTarget** target) {
+  [[nodiscard]] nsresult GetSocketThreadTarget(nsIEventTarget** target) {
     return mConnMgr->GetSocketThreadTarget(target);
   }
 
-  MOZ_MUST_USE nsresult SpeculativeConnect(nsHttpConnectionInfo* ci,
-                                           nsIInterfaceRequestor* callbacks,
-                                           uint32_t caps = 0) {
+  [[nodiscard]] nsresult SpeculativeConnect(nsHttpConnectionInfo* ci,
+                                            nsIInterfaceRequestor* callbacks,
+                                            uint32_t caps = 0,
+                                            bool aFetchHTTPSRR = false) {
     TickleWifi(callbacks);
     RefPtr<nsHttpConnectionInfo> clone = ci->Clone();
-    return mConnMgr->SpeculativeConnect(clone, callbacks, caps);
+    return mConnMgr->SpeculativeConnect(clone, callbacks, caps, nullptr,
+                                        aFetchHTTPSRR | EchConfigEnabled());
+  }
+
+  [[nodiscard]] nsresult SpeculativeConnect(nsHttpConnectionInfo* ci,
+                                            nsIInterfaceRequestor* callbacks,
+                                            uint32_t caps,
+                                            SpeculativeTransaction* aTrans) {
+    RefPtr<nsHttpConnectionInfo> clone = ci->Clone();
+    return mConnMgr->SpeculativeConnect(clone, callbacks, caps, aTrans);
   }
 
   // Alternate Services Maps are main thread only
   void UpdateAltServiceMapping(AltSvcMapping* map, nsProxyInfo* proxyInfo,
                                nsIInterfaceRequestor* callbacks, uint32_t caps,
                                const OriginAttributes& originAttributes) {
-    mConnMgr->UpdateAltServiceMapping(map, proxyInfo, callbacks, caps,
-                                      originAttributes);
+    mAltSvcCache->UpdateAltServiceMapping(map, proxyInfo, callbacks, caps,
+                                          originAttributes);
+  }
+
+  void UpdateAltServiceMappingWithoutValidation(
+      AltSvcMapping* map, nsProxyInfo* proxyInfo,
+      nsIInterfaceRequestor* callbacks, uint32_t caps,
+      const OriginAttributes& originAttributes) {
+    mAltSvcCache->UpdateAltServiceMappingWithoutValidation(
+        map, proxyInfo, callbacks, caps, originAttributes);
   }
 
   already_AddRefed<AltSvcMapping> GetAltServiceMapping(
       const nsACString& scheme, const nsACString& host, int32_t port, bool pb,
-      const OriginAttributes& originAttributes) {
-    return mConnMgr->GetAltServiceMapping(scheme, host, port, pb,
-                                          originAttributes);
+      const OriginAttributes& originAttributes, bool aHttp2Allowed,
+      bool aHttp3Allowed) {
+    return mAltSvcCache->GetAltServiceMapping(
+        scheme, host, port, pb, originAttributes, aHttp2Allowed, aHttp3Allowed);
   }
 
   //
   // The HTTP handler caches pointers to specific XPCOM services, and
   // provides the following helper routines for accessing those services:
   //
-  MOZ_MUST_USE nsresult GetStreamConverterService(nsIStreamConverterService**);
-  MOZ_MUST_USE nsresult GetIOService(nsIIOService** service);
+  [[nodiscard]] nsresult GetIOService(nsIIOService** result);
   nsICookieService* GetCookieService();  // not addrefed
   nsISiteSecurityService* GetSSService();
 
-  // callable from socket thread only
-  uint32_t Get32BitsOfPseudoRandom();
-
   // Called by the channel synchronously during asyncOpen
   void OnFailedOpeningRequest(nsIHttpChannel* chan) {
     NotifyObservers(chan, NS_HTTP_ON_FAILED_OPENING_REQUEST_TOPIC);
@@ -356,21 +357,29 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
     NotifyObservers(chan, NS_HTTP_ON_OPENING_REQUEST_TOPIC);
   }
 
+  void OnOpeningDocumentRequest(nsIIdentChannel* chan) {
+    NotifyObservers(chan, NS_DOCUMENT_ON_OPENING_REQUEST_TOPIC);
+  }
+
   // Called by the channel before writing a request
   void OnModifyRequest(nsIHttpChannel* chan) {
     NotifyObservers(chan, NS_HTTP_ON_MODIFY_REQUEST_TOPIC);
   }
 
+  // Same as OnModifyRequest but before cookie headers are written.
+  void OnModifyRequestBeforeCookies(nsIHttpChannel* chan) {
+    NotifyObservers(chan, NS_HTTP_ON_MODIFY_REQUEST_BEFORE_COOKIES_TOPIC);
+  }
+
+  void OnModifyDocumentRequest(nsIIdentChannel* chan) {
+    NotifyObservers(chan, NS_DOCUMENT_ON_MODIFY_REQUEST_TOPIC);
+  }
+
   // Called by the channel before writing a request
   void OnStopRequest(nsIHttpChannel* chan) {
     NotifyObservers(chan, NS_HTTP_ON_STOP_REQUEST_TOPIC);
   }
 
-  // Called by the channel and cached in the loadGroup
-  void OnUserAgentRequest(nsIHttpChannel* chan) {
-    NotifyObservers(chan, NS_HTTP_ON_USERAGENT_REQUEST_TOPIC);
-  }
-
   // Called by the channel before setting up the transaction
   void OnBeforeConnect(nsIHttpChannel* chan) {
     NotifyObservers(chan, NS_HTTP_ON_BEFORE_CONNECT_TOPIC);
@@ -393,7 +402,7 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
 
   // Called by channels before a redirect happens. This notifies both the
   // channel's and the global redirect observers.
-  MOZ_MUST_USE nsresult AsyncOnChannelRedirect(
+  [[nodiscard]] nsresult AsyncOnChannelRedirect(
       nsIChannel* oldChan, nsIChannel* newChan, uint32_t flags,
       nsIEventTarget* mainThreadEventTarget = nullptr);
 
@@ -403,15 +412,17 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
     NotifyObservers(chan, NS_HTTP_ON_EXAMINE_CACHED_RESPONSE_TOPIC);
   }
 
-  void OnMayChangeProcess(nsIHttpChannel* chan) {
-    NotifyObservers(chan, NS_HTTP_ON_MAY_CHANGE_PROCESS_TOPIC);
+  // Called by the channel when the transaction pump is suspended because of
+  // trying to get credentials asynchronously.
+  void OnTransactionSuspendedDueToAuthentication(nsIHttpChannel* chan) {
+    NotifyObservers(chan, "http-on-transaction-suspended-authentication");
   }
 
   // Generates the host:port string for use in the Host: header as well as the
   // CONNECT line for proxies. This handles IPv6 literals correctly.
-  static MOZ_MUST_USE nsresult GenerateHostPort(const nsCString& host,
-                                                int32_t port,
-                                                nsACString& hostLine);
+  [[nodiscard]] static nsresult GenerateHostPort(const nsCString& host,
+                                                 int32_t port,
+                                                 nsACString& hostLine);
 
   SpdyInformation* SpdyInfo() { return &mSpdyInfo; }
   bool IsH2MandatorySuiteEnabled() { return mH2MandatorySuiteEnabled; }
@@ -427,15 +438,17 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
 
   uint32_t DefaultHpackBuffer() const { return mDefaultHpackBuffer; }
 
-  bool Bug1563538() const { return mBug1563538; }
-  bool Bug1563695() const { return mBug1563695; }
-  bool Bug1562315() const { return mBug1562315; }
-  bool Bug1556491() const { return mBug1556491; }
+  static bool IsHttp3Enabled();
+  bool IsHttp3VersionSupported(const nsACString& version);
 
-  uint32_t MaxHttpResponseHeaderSize() const {
-    return mMaxHttpResponseHeaderSize;
+  static bool IsHttp3SupportedByServer(nsHttpResponseHead* aResponseHead);
+  uint32_t DefaultQpackTableSize() const { return mQpackTableSize; }
+  uint16_t DefaultHttp3MaxBlockedStreams() const {
+    return (uint16_t)mHttp3MaxBlockedStreams;
   }
 
+  const nsCString& Http3QlogDir();
+
   float FocusedWindowTransactionRatio() const {
     return mFocusedWindowTransactionRatio;
   }
@@ -446,49 +459,86 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   // took place.  Called only on the parent process and only updates
   // mLastActiveTabLoadOptimizationHit timestamp to now.
   void NotifyActiveTabLoadOptimization();
-  TimeStamp const GetLastActiveTabLoadOptimizationHit();
+  TimeStamp GetLastActiveTabLoadOptimizationHit();
   void SetLastActiveTabLoadOptimizationHit(TimeStamp const& when);
   bool IsBeforeLastActiveTabLoadOptimization(TimeStamp const& when);
 
-  bool DumpHpackTables() { return mDumpHpackTables; }
-
   HttpTrafficAnalyzer* GetHttpTrafficAnalyzer();
 
   bool GetThroughCaptivePortal() { return mThroughCaptivePortal; }
 
+  nsresult CompleteUpgrade(HttpTransactionShell* aTrans,
+                           nsIHttpUpgradeListener* aUpgradeListener);
+
+  nsresult DoShiftReloadConnectionCleanupWithConnInfo(
+      nsHttpConnectionInfo* aCI);
+
+  void MaybeAddAltSvcForTesting(nsIURI* aUri, const nsACString& aUsername,
+                                bool aPrivateBrowsing,
+                                nsIInterfaceRequestor* aCallbacks,
+                                const OriginAttributes& aOriginAttributes);
+
+  bool UseHTTPSRRAsAltSvcEnabled() const;
+
+  bool EchConfigEnabled(bool aIsHttp3 = false) const;
+  // When EchConfig is enabled and all records with echConfig are failed, this
+  // functon indicate whether we can fallback to the origin server.
+  // In the case an HTTPS RRSet contains some RRs with echConfig and some
+  // without, we always fallback to the origin one.
+  bool FallbackToOriginIfConfigsAreECHAndAllFailed() const;
+
+  // So we can ensure that this is done during process preallocation to
+  // avoid first-use overhead
+  static void PresetAcceptLanguages();
+
+  bool HttpActivityDistributorActivated();
+  void ObserveHttpActivityWithArgs(const HttpActivityArgs& aArgs,
+                                   uint32_t aActivityType,
+                                   uint32_t aActivitySubtype, PRTime aTimestamp,
+                                   uint64_t aExtraSizeData,
+                                   const nsACString& aExtraStringData);
+
  private:
   nsHttpHandler();
 
   virtual ~nsHttpHandler();
 
-  MOZ_MUST_USE nsresult Init();
+  [[nodiscard]] nsresult Init();
 
   //
   // Useragent/prefs helper methods
   //
   void BuildUserAgent();
   void InitUserAgentComponents();
+  static void PrefsChanged(const char* pref, void* self);
   void PrefsChanged(const char* pref);
 
-  MOZ_MUST_USE nsresult SetAcceptLanguages();
-  MOZ_MUST_USE nsresult SetAcceptEncodings(const char*, bool mIsSecure);
+  [[nodiscard]] nsresult SetAcceptLanguages();
+  [[nodiscard]] nsresult SetAcceptEncodings(const char*, bool mIsSecure);
 
-  MOZ_MUST_USE nsresult InitConnectionMgr();
+  [[nodiscard]] nsresult InitConnectionMgr();
 
-  void NotifyObservers(nsIHttpChannel* chan, const char* event);
+  void NotifyObservers(nsIChannel* chan, const char* event);
 
-  void SetFastOpenOSSupport();
+  friend class SocketProcessChild;
+  void SetHttpHandlerInitArgs(const HttpHandlerInitArgs& aArgs);
+  void SetDeviceModelId(const nsACString& aModelId);
 
-  void EnsureUAOverridesInit();
-
-  // Checks if there are any user certs or active smart cards on a different
-  // thread. Updates mSpeculativeConnectEnabled when done.
-  void MaybeEnableSpeculativeConnect();
+  // We only allow DNSUtils and TRRServiceChannel itself to create
+  // TRRServiceChannel.
+  friend class TRRServiceChannel;
+  friend class DNSUtils;
+  nsresult CreateTRRServiceChannel(nsIURI* uri, nsIProxyInfo* givenProxyInfo,
+                                   uint32_t proxyResolveFlags, nsIURI* proxyURI,
+                                   nsILoadInfo* aLoadInfo, nsIChannel** result);
+  nsresult SetupChannelInternal(HttpBaseChannel* aChannel, nsIURI* uri,
+                                nsIProxyInfo* givenProxyInfo,
+                                uint32_t proxyResolveFlags, nsIURI* proxyURI,
+                                nsILoadInfo* aLoadInfo, nsIChannel** result);
 
  private:
   // cached services
   nsMainThreadPtrHandle<nsIIOService> mIOService;
-  nsMainThreadPtrHandle<nsIStreamConverterService> mStreamConvSvc;
   nsMainThreadPtrHandle<nsICookieService> mCookieService;
   nsMainThreadPtrHandle<nsISiteSecurityService> mSSService;
 
@@ -497,66 +547,69 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   nsHttpAuthCache mPrivateAuthCache;
 
   // the connection manager
-  RefPtr<nsHttpConnectionMgr> mConnMgr;
+  RefPtr<HttpConnectionMgrShell> mConnMgr;
 
-  // This thread is used for performing operations that should not block
-  // the main thread.
-  nsCOMPtr<nsIThread> mBackgroundThread;
+  UniquePtr<AltSvcCache> mAltSvcCache;
 
   //
   // prefs
   //
 
-  enum HttpVersion mHttpVersion;
-  enum HttpVersion mProxyHttpVersion;
-  uint32_t mCapabilities;
+  enum HttpVersion mHttpVersion { HttpVersion::v1_1 };
+  enum HttpVersion mProxyHttpVersion { HttpVersion::v1_1 };
+  uint32_t mCapabilities{NS_HTTP_ALLOW_KEEPALIVE};
 
-  bool mFastFallbackToIPv4;
+  bool mFastFallbackToIPv4{false};
   PRIntervalTime mIdleTimeout;
   PRIntervalTime mSpdyTimeout;
   PRIntervalTime mResponseTimeout;
-  bool mResponseTimeoutEnabled;
-  uint32_t mNetworkChangedTimeout;  // milliseconds
-  uint16_t mMaxRequestAttempts;
-  uint16_t mMaxRequestDelay;
-  uint16_t mIdleSynTimeout;
-  uint16_t mFallbackSynTimeout;  // seconds
+  Atomic<bool, Relaxed> mResponseTimeoutEnabled{false};
+  uint32_t mNetworkChangedTimeout{5000};  // milliseconds
+  uint16_t mMaxRequestAttempts{6};
+  uint16_t mMaxRequestDelay{10};
+  uint16_t mIdleSynTimeout{250};
+  uint16_t mFallbackSynTimeout{5};  // seconds
 
-  bool mH2MandatorySuiteEnabled;
-  uint16_t mMaxUrgentExcessiveConns;
-  uint16_t mMaxConnections;
-  uint8_t mMaxPersistentConnectionsPerServer;
-  uint8_t mMaxPersistentConnectionsPerProxy;
+  bool mH2MandatorySuiteEnabled{false};
+  uint16_t mMaxUrgentExcessiveConns{3};
+  uint16_t mMaxConnections{24};
+  uint8_t mMaxPersistentConnectionsPerServer{2};
+  uint8_t mMaxPersistentConnectionsPerProxy{4};
 
-  bool mThrottleEnabled;
-  uint32_t mThrottleVersion;
-  uint32_t mThrottleSuspendFor;
-  uint32_t mThrottleResumeFor;
-  uint32_t mThrottleReadLimit;
-  uint32_t mThrottleReadInterval;
-  uint32_t mThrottleHoldTime;
-  uint32_t mThrottleMaxTime;
+  bool mThrottleEnabled{true};
+  uint32_t mThrottleVersion{2};
+  uint32_t mThrottleSuspendFor{3000};
+  uint32_t mThrottleResumeFor{200};
+  uint32_t mThrottleReadLimit{8000};
+  uint32_t mThrottleReadInterval{500};
+  uint32_t mThrottleHoldTime{600};
+  uint32_t mThrottleMaxTime{3000};
 
-  int32_t mSendWindowSize;
+  int32_t mSendWindowSize{1024};
 
-  bool mUrgentStartEnabled;
-  bool mTailBlockingEnabled;
-  uint32_t mTailDelayQuantum;
-  uint32_t mTailDelayQuantumAfterDCL;
-  uint32_t mTailDelayMax;
-  uint32_t mTailTotalMax;
+  bool mUrgentStartEnabled{true};
+  bool mTailBlockingEnabled{true};
+  uint32_t mTailDelayQuantum{600};
+  uint32_t mTailDelayQuantumAfterDCL{100};
+  uint32_t mTailDelayMax{6000};
+  uint32_t mTailTotalMax{0};
 
-  uint8_t mRedirectionLimit;
+  uint8_t mRedirectionLimit{10};
+
+  bool mBeConservativeForProxy{true};
 
   // we'll warn the user if we load an URL containing a userpass field
   // unless its length is less than this threshold.  this warning is
   // intended to protect the user against spoofing attempts that use
   // the userpass field of the URL to obscure the actual origin server.
-  uint8_t mPhishyUserPassLength;
+  uint8_t mPhishyUserPassLength{1};
 
-  uint8_t mQoSBits;
+  uint8_t mQoSBits{0x00};
 
-  bool mEnforceAssocReq;
+  bool mEnforceAssocReq{false};
+
+  nsCString mImageAcceptHeader;
+  nsCString mDocumentAcceptHeader;
 
   nsCString mAcceptLanguages;
   nsCString mHttpAcceptEncodings;
@@ -566,141 +619,118 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
 
   // cache support
   uint32_t mLastUniqueID;
-  uint32_t mSessionStartTime;
+  Atomic<uint32_t, Relaxed> mSessionStartTime{0};
 
   // useragent components
-  nsCString mLegacyAppName;
-  nsCString mLegacyAppVersion;
+  nsCString mLegacyAppName{"Mozilla"};
+  nsCString mLegacyAppVersion{"5.0"};
   nsCString mPlatform;
   nsCString mOscpu;
   nsCString mMisc;
-  nsCString mProduct;
+  nsCString mProduct{"Gecko"};
   nsCString mProductSub;
   nsCString mAppName;
   nsCString mAppVersion;
   nsCString mCompatFirefox;
-  bool mCompatFirefoxEnabled;
+  bool mCompatFirefoxEnabled{false};
   nsCString mCompatDevice;
   nsCString mDeviceModelId;
 
   nsCString mUserAgent;
   nsCString mSpoofedUserAgent;
   nsCString mUserAgentOverride;
-  bool mUserAgentIsDirty;  // true if mUserAgent should be rebuilt
-  bool mAcceptLanguagesIsDirty;
+  bool mUserAgentIsDirty{true};  // true if mUserAgent should be rebuilt
+  bool mAcceptLanguagesIsDirty{true};
 
-  bool mPromptTempRedirect;
+  bool mPromptTempRedirect{true};
 
   // Persistent HTTPS caching flag
-  bool mEnablePersistentHttpsCaching;
+  bool mEnablePersistentHttpsCaching{false};
 
   // for broadcasting safe hint;
-  bool mSafeHintEnabled;
-  bool mParentalControlEnabled;
+  bool mSafeHintEnabled{false};
+  bool mParentalControlEnabled{false};
 
   // true in between init and shutdown states
-  Atomic<bool, Relaxed> mHandlerActive;
+  Atomic<bool, Relaxed> mHandlerActive{false};
 
   // The value of 'hidden' network.http.debug-observations : 1;
   uint32_t mDebugObservations : 1;
 
-  uint32_t mEnableSpdy : 1;
-  uint32_t mHttp2Enabled : 1;
-  uint32_t mUseH2Deps : 1;
-  uint32_t mEnforceHttp2TlsProfile : 1;
-  uint32_t mCoalesceSpdy : 1;
-  uint32_t mSpdyPersistentSettings : 1;
-  uint32_t mAllowPush : 1;
   uint32_t mEnableAltSvc : 1;
   uint32_t mEnableAltSvcOE : 1;
   uint32_t mEnableOriginExtension : 1;
-  uint32_t mEnableH2Websockets : 1;
-  uint32_t mDumpHpackTables : 1;
 
   // Try to use SPDY features instead of HTTP/1.1 over SSL
   SpdyInformation mSpdyInfo;
 
-  uint32_t mSpdySendingChunkSize;
-  uint32_t mSpdySendBufferSize;
-  uint32_t mSpdyPushAllowance;
-  uint32_t mSpdyPullAllowance;
-  uint32_t mDefaultSpdyConcurrent;
+  uint32_t mSpdySendingChunkSize{ASpdySession::kSendingChunkSize};
+  uint32_t mSpdySendBufferSize{ASpdySession::kTCPSendBufferSize};
+  uint32_t mSpdyPushAllowance{
+      ASpdySession::kInitialPushAllowance};  // match default pref
+  uint32_t mSpdyPullAllowance{ASpdySession::kInitialRwin};
+  uint32_t mDefaultSpdyConcurrent{ASpdySession::kDefaultMaxConcurrent};
   PRIntervalTime mSpdyPingThreshold;
   PRIntervalTime mSpdyPingTimeout;
 
   // The maximum amount of time to wait for socket transport to be
   // established. In milliseconds.
-  uint32_t mConnectTimeout;
+  uint32_t mConnectTimeout{90000};
 
   // The maximum amount of time to wait for a tls handshake to be
   // established. In milliseconds.
-  uint32_t mTLSHandshakeTimeout;
+  uint32_t mTLSHandshakeTimeout{30000};
 
   // The maximum number of current global half open sockets allowable
   // when starting a new speculative connection.
-  uint32_t mParallelSpeculativeConnectLimit;
+  uint32_t mParallelSpeculativeConnectLimit{6};
 
   // For Rate Pacing of HTTP/1 requests through a netwerk/base/EventTokenBucket
   // Active requests <= *MinParallelism are not subject to the rate pacing
-  bool mRequestTokenBucketEnabled;
-  uint16_t mRequestTokenBucketMinParallelism;
-  uint32_t mRequestTokenBucketHz;     // EventTokenBucket HZ
-  uint32_t mRequestTokenBucketBurst;  // EventTokenBucket Burst
+  bool mRequestTokenBucketEnabled{true};
+  uint16_t mRequestTokenBucketMinParallelism{6};
+  uint32_t mRequestTokenBucketHz{100};    // EventTokenBucket HZ
+  uint32_t mRequestTokenBucketBurst{32};  // EventTokenBucket Burst
 
   // Whether or not to block requests for non head js/css items (e.g. media)
   // while those elements load.
-  bool mCriticalRequestPrioritization;
+  bool mCriticalRequestPrioritization{true};
 
   // TCP Keepalive configuration values.
 
   // True if TCP keepalive is enabled for short-lived conns.
-  bool mTCPKeepaliveShortLivedEnabled;
+  bool mTCPKeepaliveShortLivedEnabled{false};
   // Time (secs) indicating how long a conn is considered short-lived.
-  int32_t mTCPKeepaliveShortLivedTimeS;
+  int32_t mTCPKeepaliveShortLivedTimeS{60};
   // Time (secs) before first keepalive probe; between successful probes.
-  int32_t mTCPKeepaliveShortLivedIdleTimeS;
+  int32_t mTCPKeepaliveShortLivedIdleTimeS{10};
 
   // True if TCP keepalive is enabled for long-lived conns.
-  bool mTCPKeepaliveLongLivedEnabled;
+  bool mTCPKeepaliveLongLivedEnabled{false};
   // Time (secs) before first keepalive probe; between successful probes.
-  int32_t mTCPKeepaliveLongLivedIdleTimeS;
+  int32_t mTCPKeepaliveLongLivedIdleTimeS{600};
 
   // if true, generate NS_ERROR_PARTIAL_TRANSFER for h1 responses with
   // incorrect content lengths or malformed chunked encodings
-  FrameCheckLevel mEnforceH1Framing;
+  FrameCheckLevel mEnforceH1Framing{FRAMECHECK_BARELY};
 
   nsCOMPtr<nsIRequestContextService> mRequestContextService;
 
   // The default size (in bytes) of the HPACK decompressor table.
-  uint32_t mDefaultHpackBuffer;
+  uint32_t mDefaultHpackBuffer{4096};
 
-  // Pref for the whole fix that bug provides
-  Atomic<bool, Relaxed> mBug1563538;
-  Atomic<bool, Relaxed> mBug1563695;
-  Atomic<bool, Relaxed> mBug1562315;
-  Atomic<bool, Relaxed> mBug1556491;
+  // Http3 parameters
+  Atomic<uint32_t, Relaxed> mQpackTableSize{4096};
+  // uint16_t is enough here, but Atomic only supports uint32_t or uint64_t.
+  Atomic<uint32_t, Relaxed> mHttp3MaxBlockedStreams{10};
 
-  // The max size (in bytes) for received Http response header.
-  uint32_t mMaxHttpResponseHeaderSize;
+  nsCString mHttp3QlogDir;
 
   // The ratio for dispatching transactions from the focused window.
-  float mFocusedWindowTransactionRatio;
-
-  // We may disable speculative connect if the browser has user certificates
-  // installed as that might randomly popup the certificate choosing window.
-  Atomic<bool, Relaxed> mSpeculativeConnectEnabled;
-
-  Atomic<bool, Relaxed> mUseFastOpen;
-  Atomic<bool, Relaxed> mFastOpenSupported;
-  uint32_t mFastOpenConsecutiveFailureLimit;
-  uint32_t mFastOpenConsecutiveFailureCounter;
-  uint32_t mFastOpenStallsLimit;
-  uint32_t mFastOpenStallsCounter;
-  uint32_t mFastOpenStallsIdleTime;
-  uint32_t mFastOpenStallsTimeout;
+  float mFocusedWindowTransactionRatio{0.9f};
 
   // If true, the transactions from active tab will be dispatched first.
-  bool mActiveTabPriority;
+  bool mActiveTabPriority{true};
 
   HttpTrafficAnalyzer mHttpTrafficAnalyzer;
 
@@ -712,8 +742,8 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
 
  public:
   // Socket thread only
-  MOZ_MUST_USE nsresult SubmitPacedRequest(ATokenBucketEvent* event,
-                                           nsICancelable** cancel) {
+  [[nodiscard]] nsresult SubmitPacedRequest(ATokenBucketEvent* event,
+                                            nsICancelable** cancel) {
     MOZ_ASSERT(OnSocketThread(), "not on socket thread");
     if (!mRequestTokenBucket) {
       return NS_ERROR_NOT_AVAILABLE;
@@ -740,13 +770,15 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   void TickleWifi(nsIInterfaceRequestor* cb);
 
  private:
-  MOZ_MUST_USE nsresult
-  SpeculativeConnectInternal(nsIURI* aURI, nsIPrincipal* aPrincipal,
-                             nsIInterfaceRequestor* aCallbacks, bool anonymous);
+  [[nodiscard]] nsresult SpeculativeConnectInternal(
+      nsIURI* aURI, nsIPrincipal* aPrincipal,
+      Maybe<OriginAttributes>&& aOriginAttributes,
+      nsIInterfaceRequestor* aCallbacks, bool anonymous);
+  void ExcludeHttp2OrHttp3Internal(const nsHttpConnectionInfo* ci);
 
   // State for generating channelIds
-  uint32_t mProcessId;
-  uint32_t mNextChannelId;
+  uint32_t mProcessId{0};
+  Atomic<uint32_t, Relaxed> mNextChannelId{1};
 
   // The last time any of the active tab page load optimization took place.
   // This is accessed on multiple threads, hence a lock is needed.
@@ -757,24 +789,46 @@ class nsHttpHandler final : public nsIHttpProtocolHandler,
   // value from ipc onstoprequest arguments.  This is a sufficent way of passing
   // it down to the content process, since the value will be used only after
   // onstoprequest notification coming from an http channel.
-  Mutex mLastActiveTabLoadOptimizationLock;
+  Mutex mLastActiveTabLoadOptimizationLock{
+      "nsHttpConnectionMgr::LastActiveTabLoadOptimization"};
   TimeStamp mLastActiveTabLoadOptimizationHit;
 
-  Mutex mSpdyBlacklistLock;
+  Mutex mHttpExclusionLock MOZ_UNANNOTATED{"nsHttpHandler::HttpExclusion"};
+
  public:
-  MOZ_MUST_USE nsresult NewChannelId(uint64_t& channelId);
+  [[nodiscard]] nsresult NewChannelId(uint64_t& channelId);
+  void AddHttpChannel(uint64_t aId, nsISupports* aChannel);
+  void RemoveHttpChannel(uint64_t aId);
+  nsWeakPtr GetWeakHttpChannel(uint64_t aId);
 
-  void BlacklistSpdy(const nsHttpConnectionInfo* ci);
-  MOZ_MUST_USE bool IsSpdyBlacklisted(const nsHttpConnectionInfo* ci);
+  void ExcludeHttp2(const nsHttpConnectionInfo* ci);
+  [[nodiscard]] bool IsHttp2Excluded(const nsHttpConnectionInfo* ci);
+  void ExcludeHttp3(const nsHttpConnectionInfo* ci);
+  [[nodiscard]] bool IsHttp3Excluded(const nsACString& aRoutedHost);
+  void Exclude0RttTcp(const nsHttpConnectionInfo* ci);
+  [[nodiscard]] bool Is0RttTcpExcluded(const nsHttpConnectionInfo* ci);
 
-  virtual nsresult EnsureHSTSDataReadyNative(
-      already_AddRefed<mozilla::net::HSTSDataCallbackWrapper> aCallback)
-      override;
+  void ExcludeHTTPSRRHost(const nsACString& aHost);
+  [[nodiscard]] bool IsHostExcludedForHTTPSRR(const nsACString& aHost);
 
  private:
-  nsTHashtable<nsCStringHashKey> mBlacklistedSpdyOrigins;
+  nsTHashSet<nsCString> mExcludedHttp2Origins;
+  nsTHashSet<nsCString> mExcludedHttp3Origins;
+  nsTHashSet<nsCString> mExcluded0RttTcpOrigins;
+  // A set of hosts that we should not upgrade to HTTPS with HTTPS RR.
+  nsTHashSet<nsCString> mExcludedHostsForHTTPSRRUpgrade;
 
-  bool mThroughCaptivePortal;
+  Atomic<bool, Relaxed> mThroughCaptivePortal{false};
+
+  // The mapping of channel id and the weak pointer of nsHttpChannel.
+  nsTHashMap<nsUint64HashKey, nsWeakPtr> mIDToHttpChannelMap;
+
+  // This is parsed pref network.http.http3.alt-svc-mapping-for-testing.
+  // The pref set artificial altSvc-s for origin for testing.
+  // This maps an origin to an altSvc.
+  nsClassHashtable<nsCStringHashKey, nsCString> mAltSvcMappingTemptativeMap;
+
+  nsCOMPtr<nsIHttpActivityDistributor> mActivityDistributor;
 };
 
 extern StaticRefPtr<nsHttpHandler> gHttpHandler;
@@ -797,16 +851,33 @@ class nsHttpsHandler : public nsIHttpProtocolHandler,
   NS_DECL_NSIPROTOCOLHANDLER
   NS_FORWARD_NSIPROXIEDPROTOCOLHANDLER(gHttpHandler->)
   NS_FORWARD_NSIHTTPPROTOCOLHANDLER(gHttpHandler->)
-  NS_FORWARD_NSISPECULATIVECONNECT(gHttpHandler->)
+
+  NS_IMETHOD SpeculativeConnect(nsIURI* aURI, nsIPrincipal* aPrincipal,
+                                nsIInterfaceRequestor* aCallbacks,
+                                bool aAnonymous) override {
+    return gHttpHandler->SpeculativeConnect(aURI, aPrincipal, aCallbacks,
+                                            aAnonymous);
+  }
+
+  NS_IMETHOD SpeculativeConnectWithOriginAttributes(
+      nsIURI* aURI, JS::Handle<JS::Value> originAttributes,
+      nsIInterfaceRequestor* aCallbacks, bool aAnonymous,
+      JSContext* cx) override {
+    return gHttpHandler->SpeculativeConnectWithOriginAttributes(
+        aURI, originAttributes, aCallbacks, aAnonymous, cx);
+  }
+
+  NS_IMETHOD_(void)
+  SpeculativeConnectWithOriginAttributesNative(
+      nsIURI* aURI, mozilla::OriginAttributes&& originAttributes,
+      nsIInterfaceRequestor* aCallbacks, bool aAnonymous) override {
+    gHttpHandler->SpeculativeConnectWithOriginAttributesNative(
+        aURI, std::move(originAttributes), aCallbacks, aAnonymous);
+  }
 
   nsHttpsHandler() = default;
 
-  MOZ_MUST_USE nsresult Init();
-  virtual nsresult EnsureHSTSDataReadyNative(
-      already_AddRefed<mozilla::net::HSTSDataCallbackWrapper> aCallback)
-      override {
-    return gHttpHandler->EnsureHSTSDataReadyNative(std::move(aCallback));
-  }
+  [[nodiscard]] nsresult Init();
 };
 
 //-----------------------------------------------------------------------------
@@ -835,7 +906,6 @@ class HSTSDataCallbackWrapper final {
   std::function<void(bool)> mCallback;
 };
 
-}  // namespace net
-}  // namespace mozilla
+}  // namespace mozilla::net
 
 #endif  // nsHttpHandler_h__
